# Distribution Checker
# Simple Journal Management Module (SimpleJournal.pm)
#
# Copyright (C) 2007-2009 The Linux Foundation. All rights reserved.
#
# This program has been developed by ISP RAS for LF.
# The ptyshell tool is originally written by Jiri Dluhos <jdluhos@suse.cz>
# Copyright (C) 2005-2007 SuSE Linux Products GmbH
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# version 2 as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

package SimpleJournal;
use strict;

use Common;
#----------------------------------------------------------------------

# Class constructor
sub new {
	my ($class, $filename) = @_;
	
	( defined $filename ) or return error "Filename undefined";
	
	my $self  = {};
	
	$self->{FILENAME} = $filename;
	
	my $fh;
	open $fh, ">$filename"
		or return error "Failed to open journal for writing: '$filename': $!";
	
	$self->{FILEHANDLE} = $fh;
	
	debug_inform "SimpleJournal opened: '$filename'";

	bless $self;
	return $self;
}

sub close {
	my ($self) = @_;
	
	$self->write_line("FINISHED");
	
	close $self->{FILEHANDLE}
		or return error "Failed to close the filehandle: $!.";
		
	$self->{FILEHANDLE} = undef;
	
	return 1;
}

sub write_line {
	my ($self, $text) = @_;
	
	print {$self->{FILEHANDLE}} $text."\n";
}
#----------------------------------------------------------------------

sub test_name {
	my ($self, $test_name) = @_;
	
	$self->write_line("TEST: $test_name");
}

sub tests_expected {
	my ($self, $expected_num) = @_;
	
	$self->write_line("TESTS_EXPECTED: $expected_num");
}

sub test_case_start {
	my ($self, $tc_name) = @_;
	
	$self->write_line("TESTCASE: $tc_name");
}

sub test_description {
	my ($self, $text) = @_;
	
	foreach ( split /\n/, $text ) {
		$self->write_line(" DESCR: $_");
	}
}

sub test_comment {
	my ($self, $text) = @_;
	
	foreach ( split /\n/, $text ) {
		$self->write_line(" COMMENT: $_");
	}
}

sub test_message {
	my ($self, $text) = @_;
	
	foreach ( split /\n/, $text ) {
		$self->write_line(" MESSAGE: $_");
	}
}

sub test_case_result {
	my ($self, $res) = @_;
	
	$self->write_line("RESULT: $res");
}
#----------------------------------------------------------------------
1; # return value
