# Distribution Checker
# Alsa-t2c Test Module (Alsa_t2c_test.pm)
#
# Copyright (C) 2007-2010 The Linux Foundation. All rights reserved.
#
# This program has been developed by ISP RAS for LF.
# The ptyshell tool is originally written by Jiri Dluhos <jdluhos@suse.cz>
# Copyright (C) 2005-2007 SuSE Linux Products GmbH
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# version 2 as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

package Alsa_t2c_test;
use strict;

use Misc;
use Subshell;

use Test_common;
our @ISA = qw(Test_common); # Inherit Test_common
#----------------------------------------------------------------------

my $TC_number = 25; # Number of test cases
my $TC_n = 0; # Number of test cases executed

#----------------------------------------------------------------------

sub prepare {
	my ($self) = @_;

	# Call the parent's prepare function
	is_ok $self->Test_common::prepare() or return $Error::Last;
	
	my $testsuite_dir = $self->option('TESTSUITE_DIR') || "/opt/lsb/test/alsa-t2c";
	$self->{TESTSUITE_DIR} = $testsuite_dir;
	$self->{RESULTS_DIR} = $self->option('RESULTS_DIR') || "/opt/lsb/test/alsa-t2c/results";
	
	# Remove old results
	cmd("rm -rf ".shq($self->{RESULTS_DIR})."/*");
	
	# Call the pre-run script
	if ( -e "/opt/lsb/test/alsa-t2c/pre-run.sh" ) {
		cmd("cd /opt/lsb/test/alsa-t2c && ./pre-run.sh");
	}
	
	return 1;
}

sub run {
	my ($self) = @_;
	
	my $subshell = $self->Spawn_subshell();
	is_ok $subshell or return $Error::Last;
	
	# Change to the test suite directory
	$subshell->Settle();
	$subshell->Send( "cd ".shq($self->{TESTSUITE_DIR})."\n" );
	
	# Run the test
	$subshell->Settle();
	$subshell->Send("./run_tests.sh; exit\n");
	
	# Just wait for the test finished
	while ( my $line = $subshell->Read(2) ) {
		$self->check_progress($line);
	}
	
	# Just confirm that the subshell has finished
	$subshell->WaitForSubshell();
	
	if ( $subshell->{EXIT_CODE} ) {
		$self->report_warning(warning "subshell exited with code ".$subshell->{EXIT_CODE}
			.".\n".$subshell->get_last_lines);
	}
	
	return 1; # Ok
}

sub check_progress {
	my ($self, $line) = @_;
	
	defined $line or $line = "";
	
	if ( $line =~ /Executing tests for .*[\r\n]/ ) {
		$TC_n++;
		$self->progress($TC_n/$TC_number);
	}
}

sub after_run {
	my ($self) = @_;
	
	$self->{JOURNAL_FILE} = $self->name.".journal";
	$self->{JOURNAL_TYPE} = 'TET';
	
	# Find the journal file and copy it to where it should be.
	is_ok $self->copy_file_globbed( $self->{RESULTS_DIR}."/0*/journal", $self->journal_file )
		or $self->report_error(complain $Error::Last);

	return 1;
}

1; # return value
