# Distribution Checker
# Cmdchk Test Module (Cmdchk_test.pm)
#
# Copyright (C) 2007-2009 The Linux Foundation. All rights reserved.
#
# This program has been developed by ISP RAS for LF.
# The ptyshell tool is originally written by Jiri Dluhos <jdluhos@suse.cz>
# Copyright (C) 2005-2007 SuSE Linux Products GmbH
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# version 2 as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

package Cmdchk_test;
use strict;

use Misc;
use Subshell;

use Test_common;
our @ISA = qw(Test_common); # Inherit Test_common
#----------------------------------------------------------------------

sub prepare {
	my ($self) = @_;

	# Call the parent's prepare function
	is_ok $self->Test_common::prepare() or return $Error::Last;
	
	# Create temporary directory
	is_ok $self->create_temp_dir()
		or return error "Failed to create temporary directory.", $Error::Last;
	
	$self->{JOURNAL_FILE} = $self->name.".journal";
	$self->{JOURNAL_TYPE} = 'TET';
	
	# Options & questions

	# profile option
	my $q = 'profile';
	if ( defined $self->{OPTIONS}{$q} ) {
		if ( !defined $self->{OPTIONS}{$q}{VALUE} ) {
			if ( defined $globals->{'std_profile'} ) {
				$self->{OPTIONS}{$q}{VALUE} = $globals->{'std_profile'};
			}
		}
	}
	
	return 1;
}

sub run {
	my ($self) = @_;
	
	# Open a subshell.
	my $subshell = $self->Spawn_subshell();
	is_ok $subshell or return $Error::Last;
	
	# Go to the test temp directory
	$subshell->Settle();
	$subshell->Send("cd ".shq($self->temp_dir)."\n");
	
	# Prepare the command
	my $command = "/opt/lsb/bin/lsbcmdchk -j ".shq($self->journal_file);
	if ( defined $self->option('profile') ) {
		$command .= " -T".$self->option('profile');
	}

	$subshell->Settle();
	$subshell->Send("$command; exit\n"); # Run the test
	
	# Just wait for the test finished
	$subshell->Settle();
	$subshell->WaitForSubshell();
	
	if ( $subshell->{EXIT_CODE} ) {
		$self->report_warning(warning "subshell exited with code ".$subshell->{EXIT_CODE}
			.".\n".$subshell->get_last_lines);
	}
	
	return 1; # Ok
}

1; # return value
