# Distribution Checker
# Desktop-t2c Test Module (Desktop_t2c_test.pm)
#
# Copyright (C) 2007-2009 The Linux Foundation. All rights reserved.
#
# This program has been developed by ISP RAS for LF.
# The ptyshell tool is originally written by Jiri Dluhos <jdluhos@suse.cz>
# Copyright (C) 2005-2007 SuSE Linux Products GmbH
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# version 2 as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

package Desktop_t2c_test;
use strict;

use Misc;
use Subshell;

use Test_common;
our @ISA = qw(Test_common); # Inherit Test_common
#----------------------------------------------------------------------

my $checkpoints = undef;
#----------------------------------------------------------------------

sub prepare {
	my ($self) = @_;

	# Call the parent's prepare function
	is_ok $self->Test_common::prepare() or return $Error::Last;
	
	my $testsuite_dir = $self->option('TESTSUITE_DIR') || "/opt/lsb/test/desktop-t2c";
	$self->{TESTSUITE_DIR} = $testsuite_dir;
	$self->{RESULTS_DIR} = $self->option('RESULTS_DIR') || "/opt/lsb/test/desktop-t2c/results";
	
	# Remove old results
	cmd("rm -rf ".shq($self->{RESULTS_DIR})."/*");
	
	# Call the pre-run script
	if ( -e "/opt/lsb/test/desktop-t2c/pre-run.sh" ) {
		cmd("cd /opt/lsb/test/desktop-t2c && LC_ALL=C ./pre-run.sh");
	}
	
	init_checkpoint_data($self->{VERSION});

	return 1;
}

sub run {
	my ($self) = @_;
	
	my $subshell = $self->Spawn_subshell('tester');
	is_ok $subshell or return $Error::Last;
	
	# Change to the test suite directory
	$subshell->Settle();
	$subshell->Send( "cd ".shq($self->{TESTSUITE_DIR})."\n" );
	
	# Run the test
	$subshell->Settle();
	$subshell->Send("./run_tests.sh; exit\n");
	
	# Just wait for the test finished
	while ( my $line = $subshell->Read(2) ) {
		$self->check_progress($line);
	}
	
	# Just confirm that the subshell has finished
	$subshell->WaitForSubshell();
	
	if ( $subshell->{EXIT_CODE} ) {
		$self->report_warning(warning "subshell exited with code ".$subshell->{EXIT_CODE}
			.".\n".$subshell->get_last_lines);
	}
	
	return 1; # Ok
}

sub check_progress {
	my ($self, $line) = @_;
	
	defined $line or $line = "";
	
	if ( $line =~ /^Executing tests for (.*)[\r\n]/ ) {
		my $testcase = $1;
		$self->progress_profiled($checkpoints, $testcase);
		return;
	}
	# else:
	$self->progress_profiled();
}

sub after_run {
	my ($self) = @_;

	$self->{JOURNAL_FILE} = $self->name.".journal";
	$self->{JOURNAL_TYPE} = 'TET';
	
	# Find the journal file and copy it to where it should be.
	is_ok $self->copy_file_globbed( $self->{RESULTS_DIR}."/0*/journal", $self->journal_file )
		or $self->report_error(complain $Error::Last);
	
	return 1;
}

sub init_checkpoint_data {
	my ($ver) = @_;
	
	# $ver <= 3.2.*
	if ( $ver lt "4.0" ) {
		$checkpoints = [
			[ 1, 'AtkObject_parent_child' ],
			[ 7, 'AtkRegistry' ],
			[ 13, 'AtkComponent_focus' ],
			[ 20, 'FcLangSet' ],
			[ 26, 'glib_quarks' ],
			[ 36, 'glib_keyed_data_lists' ],
			[ 42, 'glib_messagelog' ],
			[ 52, 'glib_memchunks' ],
			[ 87, 'glib_singly_linked_lists' ],
			[ 95, 'glib_commandline_option_parser' ],
		];
	}
	else { # 4.0
		$checkpoints = [
			[ 7, 'AtkObjectFactory' ],
			[ 13, 'AtkText_attributes_self' ],
			[ 20, 'FcObjectSet' ],
			[ 26, 'opentype_validation' ],
			[ 32, 'ModuleInterface' ],
			[ 38, 'Rendering' ],
			[ 48, 'glib_memchunks' ],
			[ 57, 'glib_arrays' ],
			[ 63, 'glib_singly_linked_lists' ],
			[ 88, 'glib_strings' ],
			[ 95, 'gobject_gtype_instance' ],
		];
	}
}


1; # return value
