# Distribution Checker
# Desktop Test Module (Desktop_test.pm)
#
# Copyright (C) 2007-2009 The Linux Foundation. All rights reserved.
#
# This program has been developed by ISP RAS for LF.
# The ptyshell tool is originally written by Jiri Dluhos <jdluhos@suse.cz>
# Copyright (C) 2005-2007 SuSE Linux Products GmbH
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# version 2 as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

package Desktop_test;
use strict;

use Misc;
use Subshell;

use Test_common;
our @ISA = qw(Test_common); # Inherit Test_common

use X11_common;
#----------------------------------------------------------------------

sub check {
	my ($self) = @_;

	# Call the parent's check() function
	is_ok $self->Test_common::check() or return $Error::Last;

	if ( version_filter("< 3.2", $self->{VERSION}) ) {
	# For LSB 3.1 and older releases
		# Check Xvfb
		does_shell_know ("Xvfb")
			or return error "Can't find the 'Xvfb' tool required by "
					.$self->name." test."
					." Note that lsb-xvfb can't be used instead Xvfb, because its path differs."
					." Please install Xvfb.";
	}
	
	return 1;
}

sub prepare {
	my ($self) = @_;
	
	# Call the parent's prepare function
	is_ok $self->Test_common::prepare() or return $Error::Last;
	
	( defined $self->{OPTIONS} )
		or return error "Answers are undefined"; # Should not happen
	
	my $testsuite_dir = $self->option('TESTSUITE_DIR') || "/opt/lsb/test/desktop";
	$self->{TESTSUITE_DIR} = $testsuite_dir;
	
	my $q;
	$q = 'X11_FONT_PATH';
	if ( defined $self->{OPTIONS}{$q} ) {
		if ( !defined $self->{OPTIONS}{$q}->{VALUE} ) {
			my $x11_fonts_dir = X11_common::guess_font_dir();
			if ( is_ok($x11_fonts_dir) && $x11_fonts_dir ) {
				$self->{OPTIONS}{$q}->{VALUE} = $x11_fonts_dir."/misc";
			}
			else {
				$self->{OPTIONS}{$q}->{ERROR} = error $self->name." test requires certain X11 fonts to be installed.", $Error::Last;

				if ( version_filter("< 3.2", $self->{VERSION}) ){
					return $self->{OPTIONS}{$q}->{ERROR};
				}
			}
		}
	}
	
	$q = 'DESKTOP_ENVIRONMENT';
	if ( defined $self->{OPTIONS}{$q} ) {
		if ( !defined $self->{OPTIONS}{$q}->{VALUE} ) {
			$self->{OPTIONS}{$q}->{VALUE} = X11_common::guess_DE();
		}
	}
	
	if ( $globals->{'cert_mode'} && version_filter("LSB >= 3.2", $globals->{'standard'}) ) {
		$self->{OPTIONS}{'with_qt4'}->{VALUE} = "desktop_qt4";
	}
	
	$q = 'XVFB_DISPLAY';
	if ( defined $self->{OPTIONS}{$q} ) {
		if ( !defined $self->{OPTIONS}{$q}->{VALUE} ) {
			$self->{OPTIONS}{$q}->{VALUE} = X11_common::guess_display();
		}
	}
	
	cmd ("killall Xvfb"); # Should be done before calling guess_display()
	sleep 1;
	
	# Remove old config
	cmd("rm -f ".shq($testsuite_dir)."/.configured");
	
	# Remove old results
	cmd("rm -rf ".shq($testsuite_dir)."/gtkvts/results/*");
	cmd("rm -rf ".shq($testsuite_dir)."/qt3/results/*");
	cmd("rm -rf ".shq($testsuite_dir)."/fontconfig/results/*");
	cmd("rm -rf ".shq($testsuite_dir)."/xft/results/*");
	cmd("rm -rf ".shq($testsuite_dir)."/xrender/results/*");
	cmd("rm -rf ".shq($testsuite_dir)."/freetype/results/*");
	cmd("rm -f ".shq($testsuite_dir)."/xml/journal.libxml2");
	cmd("rm -f ".shq($testsuite_dir)."/libpng/journal.pngtest");
	cmd("rm -f ".shq($testsuite_dir)."/Qt/journal.qt");
	cmd("rm -f ".shq($testsuite_dir)."/cairo/journal.cairo");
	
	# Prepare list of available subtests
	for my $subname ( "libpng", "qt3", "fontconfig", "gtkvts", "xml",
			"qt4", "freetype", "xrender", "xft", "cairo" )
	{
		$self->{JOURNALS}{$subname} = {};
		
		if ( in_array($subname, "libpng", "fontconfig", "gtkvts", "xml") ) {
			$self->{JOURNALS}{$subname}{REQUIRED} = 1;
		}
		if ( in_array($subname, "qt4") ) {
			if ( version_filter("LSB >= 3.2 | Moblin", $globals->{'standard'})
			   || $self->option('with_qt4', "desktop_qt4") )
			{
				$self->{JOURNALS}{$subname}{REQUIRED} = 1;
			}
		}
		if ( in_array($subname, "freetype" ) ) {
			if ( version_filter("LSB >= 3.2 | Moblin", $globals->{'standard'}) ) {
				$self->{JOURNALS}{$subname}{REQUIRED} = 1;
			}
		}
		if ( in_array($subname, "xrender", "xft") ) {
			# 'xrender' and 'xft' are trial in LSB 3.2
			if ( version_filter("LSB > 3.2 | Moblin", $globals->{'standard'}) ) {
				$self->{JOURNALS}{$subname}{REQUIRED} = 1;
			}
		}
		if ( in_array($subname, "cairo") ) {
			if ( version_filter("LSB >= 4.0 | Moblin", $globals->{'standard'}) ) {
				$self->{JOURNALS}{$subname}{REQUIRED} = 1;
			}
		}
		if ( in_array($subname, "qt3") ) {
			if ( version_filter("LSB < 5.0 | Moblin", $globals->{'standard'}) ) {
				$self->{JOURNALS}{$subname}{REQUIRED} = 1;
			}
		}
	}
	
	return 1;
}

sub run {
	my ($self) = @_;
	
	# Check whether the '-a' option is supported
	my $automatic_option = 
		( 0 == cmd( shq($self->{TESTSUITE_DIR}."/run_tests")." -h | grep -e '\\-a '") );
	
	my $subshell = $self->Spawn_subshell();
	is_ok $subshell or return $Error::Last;
	
	# Change to the test suite directory
	$subshell->Settle();
	$subshell->Send( "cd ".shq($self->{TESTSUITE_DIR})."\n" );
	
	# Prepare the command
	my $command = "./run_tests";
	$command .= " -a" if $automatic_option;
	if ( $self->option('with_qt4', "desktop_qt4") ) {
		$command .= " -m QT4";
	}
	
	$subshell->Settle();
	$subshell->Send("$command; exit\n"); # Run the test
	
	my $questions_wait_time = 60*10; # In seconds
	my $questions_deadline = time() + $questions_wait_time;
	
	my $started = 0;
	
	my $line;
	DIALOG_LOOP:
	while ( $line = $subshell->Read($questions_wait_time) ) {
		last if !defined $line; # required for 'redo'
		
		if ( $line =~ /Starting X Virtual Frame Buffer/ ) {
			$started = 1;
			last;
		}
		
		# Answer the prompts
		
		$subshell->Settle($line) or redo DIALOG_LOOP;
		
		# Formalized questions
		foreach my $opt ( values %{$self->{OPTIONS}} ) {
			next if !defined $opt->{PRE};
			
			my $pre = $opt->{PRE};
			if ( $line =~ /$pre/ ) {

				if ( !defined $opt->{VALUE} ) {
					is_ok $subshell->ExpectLogout() or $self->report_error(complain $Error::Last);
					return $opt->{ERROR} if is_err($opt->{ERROR});
					return error $self->name.": can't answer question: '".$line."'.";
				}
				$subshell->Send( $opt->{VALUE}."\n" );
				next DIALOG_LOOP;
			}
		}
		
	} continue {
		
		# Check the questions timeout
		if ( time() > $questions_deadline ) {
			inform "Time for questions is over.";
			is_ok $subshell->ExpectLogout();
			return error "Timeout", $Error::Last;
		}
	}
	if ( $subshell->{TIMEOUT} ) { inform "!!! Warning: timeout at subshell"; }
	
	# Just wait for the test finished
	$subshell->WaitForSubshell();
	
	if ( $subshell->{EXIT_CODE} ) {
		$self->report_warning(warning "subshell exited with code ".$subshell->{EXIT_CODE}
			.".\n".$subshell->get_last_lines);
	}
	
	if ( !$started ) {
		return error "The test hasn't started";
	}
	
	return 1; # Ok
}

sub after_run {
	my ($self) = @_;
	
	return 1 if !$self->{JOURNALS};
	
	my $testsuite_dir = $self->{TESTSUITE_DIR};
	
	my $journal_location = {
			"gtkvts"     => $testsuite_dir."/gtkvts/results/0*/journal",
			"qt3"        => $testsuite_dir."/qt3/results/0*/journal",
			"fontconfig" => $testsuite_dir."/fontconfig/results/0*/journal",
			"xft"        => $testsuite_dir."/xft/results/0*/journal",
			"xrender"    => $testsuite_dir."/xrender/results/0*/journal",
			"freetype"   => $testsuite_dir."/freetype/results/0*/journal",
			"xml"        => $testsuite_dir."/xml/journal.libxml2",
			"libpng"     => $testsuite_dir."/libpng/journal.pngtest",
			"qt4"        => $testsuite_dir."/Qt/journal.qt",
			"cairo"      => $testsuite_dir."/cairo/journal.cairo",
		};

	my $packed = "";
	foreach my $subname ( sort keys %{$self->{JOURNALS}} ) {
		my $obj = $self->{JOURNALS}{$subname};
		if ( $obj ) {
			my $journal_file_name = "desktop_".$subname.".journal";
			my $journal_file = $globals->{'test_result_dir'}."/".$journal_file_name;
			
			my $res = $self->copy_file_globbed( $journal_location->{$subname}, $journal_file );
			if ( !is_ok($res) ) {
				if ( $obj->{REQUIRED} ) {
					$self->report_error(complain $Error::Last);
				} else {
					# Not required and no file
					next if ( !-f $journal_file );
				}
			}

			# Pack $self->{JOURNALS}
			# The JOURNALS should be string to be saved to the .info file.
			$packed .= $subname.":";
			$packed .= "FILE=".$journal_file.",KIND=TET";
			if ( !$obj->{REQUIRED} ) {
				$packed .= ",TRIAL_USE=1";
			}
			$packed .= ";";
		}
	}
	$self->{JOURNALS} = $packed;
	
	return 1;
}

sub make_report {
	my ($self) = @_;
	
	( $self->{JOURNALS} ) or return error("No journals");
	
	# Unpack $self->{JOURNALS}: "key=value,key=value,..."
	my $journals = {};
	my @subtests = ();
	for my $s ( split /;/, $self->{JOURNALS} ) {
		my ($subname, $data) = split /:/, $s, 2;
		foreach my $pair ( split /,/, $data ) {
			my ($key, $value) = split /=/, $pair;
			$journals->{$subname}{$key} = $value;
		}
		push @subtests, $subname;
	}
	
	# Make journals table
	my $journals_toc = {};
	my $stats = {};
	my @verdicts = ();
	my $report = "";
	
	foreach my $subname ( @subtests ) {
		my $obj = $journals->{$subname};
		
		# Add an anchor for journals TOC
		$report .= '<a name="'.html($subname).'"></a>'."\n";
		# Title
		my $title = "desktop-".$subname;
		$report .= '<h2>'.html($title).'</h2>'."\n";
		
		my $jstruct = { %$obj };
		if ( !$obj->{FILE} ) {
			local $Error::Debug = 0;
			$report .= Report::ErrMessage(complain "$subname: Journal file undefined");
			next;
		}
		$jstruct->{TESTSUITE} = $title;
		$jstruct->{TESTSUITE_VER} = $self->{VERSION};
		$jstruct->{STANDARD} = $globals->{'standard'};
		$jstruct->{ARCHITECTURE} = $host_info->{'architecture'};
		
		# Print a link to the journal
		$report .= '<p>Open the test '.Report::local_href( $jstruct->{FILE}, 'journal' ).'.</p>';
		
		is_ok Report::ParseJournal( $jstruct )
			or do {
				local $Error::Debug = 0;
				$report .= Report::ErrMessage( complain "Failed to parse journal: ".$jstruct->{FILE}, $Error::Last );
				push @verdicts, 'run_problems';
				next;
			};
		
		if ( $subname eq 'cairo' ) {
			# cairo journal is malformed and it cause many INFO_TABLES to appear. Truncate them.
			$#{$jstruct->{INFO_TABLES}} = 1 if $jstruct->{INFO_TABLES} && $#{$jstruct->{INFO_TABLES}} > 2;
		}

		if ( $jstruct->{STATS} ) {
			for my $stat_key ( %{$jstruct->{STATS}} ) {
				$stats->{$stat_key} += ($jstruct->{STATS}{$stat_key} or 0);
			}
		}

		# Print the errors table
		$report .= Report::JournalReport($jstruct);
		# Print the statistics
		$report .= Report::print_stats($jstruct->{STATS});
		
		my $journal_verdict = Report::verdict( $jstruct->{STATS} );
		push @verdicts, $journal_verdict;
		push @{$journals_toc->{ROWS}}, {
				LEFT => '<a href="#'.html($subname).'">'.html($title).'</a>',
				RIGHT => Report::verdict_text($journal_verdict),
				BG_COLOR => Report::verdict_color($journal_verdict),
			};
	}

	$self->{REPORT_STATS} = $stats;
	$self->{VERDICT} = Report::overall_verdict(@verdicts);
	
	$journals_toc->{TITLE} = 'Test journals';
	$self->report( Report::print_info_table($journals_toc) );
	
	$self->report($report);
	
	return 1;
}

sub Cleanup {
	my ($self) = @_;
	
	cmd("killall Xvfb");
}

1; # return value
