# Distribution Checker
# Libchk Test Module (Libchk_test.pm)
#
# Copyright (C) 2007-2009 The Linux Foundation. All rights reserved.
#
# This program has been developed by ISP RAS for LF.
# The ptyshell tool is originally written by Jiri Dluhos <jdluhos@suse.cz>
# Copyright (C) 2005-2007 SuSE Linux Products GmbH
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# version 2 as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

package Libchk_test;
use strict;

use Misc;
use Subshell;

use Test_common;
our @ISA = qw(Test_common); # Inherit Test_common
#-----------------------------------------------------------------------

my $checkpoints = undef;

#-----------------------------------------------------------------------

sub prepare {
	my ($self) = @_;

	# Call the parent's prepare function
	is_ok $self->Test_common::prepare() or return $Error::Last;
	
	$self->{JOURNAL_FILE} = $self->name.".journal";
	$self->{JOURNAL_TYPE} = 'TET';
	
	# Options & questions

	# profile option
	my $q = 'profile';
	if ( defined $self->{OPTIONS}{$q} ) {
		if ( !defined $self->{OPTIONS}{$q}{VALUE} ) {
			if ( defined $globals->{'std_profile'} ) {
				$self->{OPTIONS}{$q}{VALUE} = $globals->{'std_profile'};
			}
		}
		elsif ( $self->{OPTIONS}{$q}{VALUE} =~ s/,qt4// ) {
			$self->{OPTIONS}{'Qt'}{VALUE} = 1;
		}
	}

	my $run_script = $self->option('RUN_SCRIPT') || "/opt/lsb/bin/lsblibchk";
	$self->{RUN_SCRIPT} = $run_script;
	
	init_checkpoint_data($self->{VERSION});
	
	return 1;
}

sub run {
	my ($self) = @_;
	
	my $subshell = $self->Spawn_subshell();
	is_ok $subshell or return $Error::Last;

	( $self->{RUN_SCRIPT} ) or return error "Undefined run script";
	
	# Prepare the command
	my $command = $self->{RUN_SCRIPT}." -j ".shq($self->journal_file);
	if ( defined $self->option('profile') ) {
		$command .= " -T".$self->option('profile');
	}
	if ( defined $self->option('Qt') ) {
		$command .= " -M LSB_Toolkit_Qt";
	}
	
	$subshell->Settle();
	$subshell->Send("$command; exit\n"); # Run the test
	
	$subshell->Settle();
	
	# Just wait for the test finished
	while ( my $line = $subshell->Read(2) ) {
		$self->check_progress($line);
	}
	
	# Just confirm that the subshell has finished
	$subshell->WaitForSubshell();
	
	if ( $subshell->{EXIT_CODE} ) {
		$self->report_warning(warning "subshell exited with code ".$subshell->{EXIT_CODE}
			.".\n".$subshell->get_last_lines);
	}
	
	return 1; # Ok
}

sub check_progress {
	my ($self, $line) = @_;
	
	defined $line or $line = "";
	
	if ( $line =~ /^Checking symbols in (\S+).*[\r\n]/ ) {
		my $testcase = $1;
		$self->progress_profiled($checkpoints, $testcase);
		return;
	}
	# else:
	$self->progress_profiled();
}

sub init_checkpoint_data {
	my ($ver) = @_;

	# TODO: depending on $self->option('profile'), $self->option('Qt');

	if ( $ver lt "3.1" ) { # 3.0.3-1, LSB 3.0
		$checkpoints = [
			[ 75, '/lib/ld-lsb.so.3' ],
			[ 100, '/lib/i686/cmov/libutil.so.1' ],
		];
	}
	elsif ( $ver ge "3.1" && $ver lt "3.2" ) { # 3.1.1-3, LSB 3.1
		$checkpoints = [
			[ 13, '/lib/ld-lsb.so.3' ],
			[ 77, '/usr/lib/libQtCore.so.4' ],
			[ 95, '/usr/lib/libQtNetwork.so.4' ],
		];
	}
	elsif ( $ver ge "3.2" && $ver lt "4" ) { # 3.2.2-1, LSB 3.2
		$checkpoints = [
			[ 12, '/lib/ld-lsb.so.3' ],
			[ 66, '/usr/lib/libQtCore.so.4' ],
			[ 91, '/usr/lib/libQtNetwork.so.4' ],
		];
	}
	else { # 4.0.0-2, LSB 4.0
		$checkpoints = [
			[ 16, '/lib/ld-lsb.so.3' ],
			[ 24, '/lib/i686/cmov/libpthread.so.0' ],
			[ 68, '/usr/lib/libQtCore.so.4' ],
			[ 96, '/usr/lib/libQtNetwork.so.4' ],
		];
	}
}
1; # return value
