# Distribution Checker
# Xpdf FVT Part Module (Xpdf_test.pm)
#
# Copyright (C) 2007-2009 The Linux Foundation. All rights reserved.
#
# This program has been developed by ISP RAS for LF.
# The ptyshell tool is originally written by Jiri Dluhos <jdluhos@suse.cz>
# Copyright (C) 2005-2007 SuSE Linux Products GmbH
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# version 2 as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

package Xpdf_test;
use strict;

use Misc;
use SimpleJournal;

use Test_common;
our @ISA = qw(Test_common); # Inherit Test_common
#----------------------------------------------------------------------

sub prepare {
	my ($self) = @_;

	# 1. Download the lsb-xpdf application package that corresponds to the LSB
	#    version and processor architecture for which certification is being 
	#    sought from the Download Page. 
	# 2. Install package on system.
	# NOTE: Application MUST be installed without use of --nodeps option,
	#  the dependency checks are part of the test.
	
	# Download and install required packages
	is_ok $self->Test_common::prepare() or return $Error::Last;
	
	# Create temporary directory
	is_ok $self->create_temp_dir()
		or return error "Failed to create temporary directory.", $Error::Last;
	
	# Change to /tmp directory and create a new directory called lsb-xpdf-tests
	# Change to the /tmp/lsb-xpdf-tests directory
	chdir( $self->temp_dir )
		or return error "Failed to chdir to '".$self->temp_dir."': $!";
	
	# Download two xpdf test files into the /tmp/lsb-xpdf-tests directory from here:
	# test1.pdf and test2.pdf
	is_ok $self->copy_files_from_reqs_to( $self->temp_dir )
		or return error "Failed to prepare test files.", $Error::Last;
	
	$self->{JOURNAL_FILE} = $self->name.".simplejournal";
	$self->{JOURNAL_TYPE} = 'SIMPLE';
	
	return 1;
}

sub run {
	my ($self) = @_;
	
	my $result;

	my $simpjournal = SimpleJournal->new( $self->journal_file );
	is_ok($simpjournal) or return error "Failed to open journal.", $Error::Last;
	
	$simpjournal->test_name( $self->name );
	
	my $tests_expected = 4;
	$simpjournal->tests_expected( $tests_expected ); # Number of tests
	
	### Functional Verification Test for lsb-apache ###
	# http://www.linuxfoundation.org/appbat/fvt-3.0.0/lsb-xpdf_fvt.html
	# http://www.linuxfoundation.org/appbat/fvt-3.1.0/lsb-xpdf_fvt.html
	# http://www.linuxfoundation.org/appbat/fvt-3.2.0/lsb-xpdf_fvt.html
	
	###  Check installation
	$simpjournal->test_case_start("Check installation");
	$simpjournal->test_description(<<HEREDOC);
- Expected result: application should be installed to 
/opt/lsb/appbat
/etc/opt/lsb/appbat
HEREDOC
	
	$result = "passed";
	for my $file ( "/opt/lsb/appbat/bin/xpdf",
				"/etc/opt/lsb/appbat/xpdfrc" )
	{
		if ( !-e $file ) {
			$result = "failed";
			$simpjournal->test_comment("'$file' is expected to exist after installation, but not found.");
		}
	}
	
	$simpjournal->test_case_result($result);
	#---------
	
	$simpjournal->test_case_start("pdfinfo");
	$simpjournal->test_description(<<HEREDOC);
Run the command /opt/lsb/appbat/bin/pdfinfo test2.pdf 
- Expected result: Information about the test2.pdf file will be displayed.
It will look something like this: 
    Producer:     GNU Ghostscript 7.05
    Tagged:       no
    Pages:        1
    Encrypted:    no
    Page size:    612 x 792 pts (letter)
    File size:    16163 bytes
    Optimized:    no
    PDF version:  1.2
HEREDOC

	# Run the command /opt/lsb/appbat/bin/pdfinfo test2.pdf
	my $cmd = "/opt/lsb/appbat/bin/pdfinfo test2.pdf";
	my $msg = capture( "$cmd 2>&1" );
	$result = $?;
	if ( $result != 0 ) {
		$simpjournal->test_comment("The command has returned non-zero code: $result");
		$simpjournal->test_message("%> $cmd");
		$simpjournal->test_message($msg);
	}
	# - Expected result: Information about the test2.pdf file will be displayed.
	# It will look something like this: 
	$simpjournal->test_message( $msg );
	$result = 1;
	foreach my $regexp ( q{Producer:\s+GNU Ghostscript\b},
				q{Tagged:\s+no\b},
				q{Pages:\s+1\b},
				q{Encrypted:\s+no},
				q{Page size:\s+\d+ x \d+ },
				q{File size:\s+\d+ bytes},
				q{Optimized:\s+no},
				q{PDF version:},
		 ) 
	{
		if ( $msg !~ /$regexp/ ) {
			$simpjournal->test_comment("Expected: /$regexp/");
			$result = 0;
		}
	}
	$simpjournal->test_case_result($result ? "passed" : "failed");
	#---------
	
	$simpjournal->test_case_start("pdftotext");
	$simpjournal->test_description(<<HEREDOC);
Run the command /opt/lsb/appbat/bin/pdftotext test1.pdf 
- Expected result: The test1.pdf file will be converted to text and saved
 in the test1.txt file. If you wish, display it with the "more" command.
HEREDOC
	# Run the command /opt/lsb/appbat/bin/pdftotext test1.pdf
	$cmd = "/opt/lsb/appbat/bin/pdftotext test1.pdf";
	$msg = capture( "$cmd 2>&1" );
	$result = $?;
	if ( $result != 0 ) {
		$simpjournal->test_comment("The command has returned non-zero code: $result");
		$simpjournal->test_message("%> $cmd");
		$simpjournal->test_message($msg);
	}
	# - Expected result: The test1.pdf file will be converted to text and saved
	# in the test1.txt file.
	my $result_file = "test1.txt";
	my $output = read_file( $result_file );
	if ( !is_ok($output) ) {
		$simpjournal->test_comment("Can't find file '$result_file'");
		$simpjournal->test_case_result("failed");
	} else {
		my $result = 1;
		foreach my $regexp ( q{Mission Statement}, )  # TODO: more matchers
		{
			if ( $output !~ /$regexp/ ) {
				$simpjournal->test_comment("Expected: /$regexp/");
				$result = 0;
			}
		}
		if ( !$result ) {
			my $output_file = $self->name.".output1";
			$self->copy_file( $self->temp_dir."/".$result_file, 
								$globals->{'test_result_dir'}."/".$output_file );
			$simpjournal->test_comment("\nSee the '$result_file' file ".
						Report::local_href("results/".$output_file, "here").".");
		}
		$simpjournal->test_case_result($result ? "passed" : "failed");
	}
	#---------



	$simpjournal->test_case_start("pdftops");
	$simpjournal->test_description(<<HEREDOC);
Run the command /opt/lsb/appbat/bin/pdftops test2.pdf 
- Expected result: The test2.pdf file will be converted to Postscript and saved
 in the test2.ps file.
HEREDOC
	# Run the command /opt/lsb/appbat/bin/pdftops test2.pdf
	$cmd = "/opt/lsb/appbat/bin/pdftops test2.pdf";
	$msg = capture( "$cmd 2>&1" );
	$result = $?;
	if ( $result != 0 ) {
		$simpjournal->test_comment("The command has returned non-zero code: $result");
		$simpjournal->test_message("%> $cmd");
		$simpjournal->test_message($msg);
	}
	# - Expected result: The test2.pdf file will be converted to Postscript and saved
	# in the test2.ps file.
	$result_file = "test2.ps";
	$output = read_file( $result_file );
	if ( !is_ok($output) ) {
		$simpjournal->test_comment("Can't find file '$result_file'");
		$simpjournal->test_case_result("failed");
	} else {
		my $result = 1;
		foreach my $regexp ( q{%%Creator: xpdf/pdftops},
						q{The Free Standards Group} )  # TODO: more matchers
		{
			if ( $output !~ /$regexp/ ) {
				$simpjournal->test_comment("Expected: /$regexp/");
				$result = 0;
			}
		}
		if ( !$result ) {
			my $output_file = $self->name.".output2";
			$self->copy_file( $self->temp_dir."/".$result_file, 
						$globals->{'test_result_dir'}."/".$output_file );
			$simpjournal->test_comment("\nSee the '$result_file' file ".
						Report::local_href("results/".$output_file, "here").";");
		}
		$simpjournal->test_case_result($result ? "passed" : "failed");
	}
	#---------
	
	$simpjournal->close();
	
	return 1; # Ok
}

#----------------------------------------------------------------------
1; # return value
