<?
/*
# Distribution Checker
# Standalone web-page displaying result directory (auto_dtk_results.php)
#
# Copyright (C) 2007-2008 The Linux Foundation. All rights reserved.
#
# This program has been developed by ISP RAS for LF.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# version 2 as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*/

// SETTINGS
$result_dir = '/var/opt/lsb/test/manager/results';

# Uncomment the following line and set up a correct path to jdiff.pl utility in it to enable result comparison feature.
//$utils_dir = '/opt/lsb/test/manager/utils';

# You should make the result dir accessible via HTTP,
# and write its address into the following string:
$www_root = 'http://yourhost.org/results';

# This page
$www_this = 'http://yourhost.org/auto_dtk_results.php';

// FUNCTIONS

// Escapes data for safe printing in the HTML code.
function html($text) {
	return htmlspecialchars($text, ENT_QUOTES);
}

function get_results() {
    global $result_dir;
    // Read the result dir
    $results = array();
    $handle = opendir($result_dir) or die('Can\'t open directory with results.');
    
    while ( false !== ( $file = readdir($handle) )  ) {
        if ($file == "." || $file == "..")  continue;
        if ( !is_dir("$result_dir/$file") ) continue;
        
        // Read INFO file
        $info_lines = @file ( "$result_dir/$file/INFO" );
        if ( $info_lines === False ) continue;
        $summary = '';
        while ( $line = each($info_lines) ) {
            $line = $line['value'];
            if ( $line == "\n" ) break; // empty line preceedes comment.
            if ( preg_match("/^Verdict(:|-\S+)\s+(.*)/", $line, $matched) ) {
                $color = "";
                if ( $matched[2] == "Success" || $matched[2] == "Passed" ) {
                    $color = "#007000";
                }
                if ( $matched[2] == "Warning" ) {
                    $color = "#706000";
                }
                elseif ( $matched[2] == "Failed" || $matched[2] == "There are problems" ) {
                    $color = "#C00000";
                }
                
                if ( $color ) {
                    $summary .= '<span style="color:'.$color.'">'.'<b>'.html($line).'</b>'.'</span>'.'<br />';
                } else {
                    $summary .= '<b>'.html($line).'</b>'.'<br />';
                }
            }
            else {
                $summary .= html($line).'<br />';
            }
        }
        // Rest lines is a comment
        $comment = '';
        while ( $line = each($info_lines) ) {
            $line = $line['value'];
            $comment .= $line;
        }
        
        if ( preg_match( '/^(\w+)-([\w-]+)-(\d+)-(\d+)-(\d+)-(\d+)h-(\d+)m-(\d+)s$/'
                , $file, $matches )  ) {
            $arch = $matches[1];
            $host = $matches[2];
            // Date/Time
            $dt['y'] = $matches[3];
            $dt['m'] = $matches[4];
            $dt['d'] = $matches[5];
            $dt['h'] = $matches[6];
            $dt['mi'] = $matches[7];
            $dt['s'] = $matches[8];
        }
        
        $result = array (
            'name' => $file,
            'summary' => $summary,
            'comment' => $comment,
            'arch' => $arch,
            'host' => $host,
            'dt' => $dt,
        );
        $sort_key = $dt['y'].$dt['m'].$dt['d'].$dt['h'].$dt['mi'].$dt['s'];
        $results[$sort_key] = $result;
    }
    closedir($handle);
    
    return $results;
}

if ( isset($_GET['goto']) ) {
    $link_addr = $www_root."/".$_GET['goto'];
    if ( file_exists($result_dir."/".$_GET['goto']."/test_log.html") ) {
        $link_addr .= "/test_log.html";
    }
    elseif ( file_exists($result_dir."/".$_GET['goto']."/report.htm") ) {
        $link_addr .= "/report.htm";
    }
    header("Location: ".html($link_addr));
    exit;
}

if ( isset($_POST['compare']) ) {
    if ( !isset($utils_dir) ) {
        exit;
    }
    $to_compare = array();
    foreach ( $_POST as $key => $value ) {
        if ( file_exists($result_dir."/".$key) ) {
            $to_compare[] = $result_dir."/".$key;
        }
    }
    
    passthru( $utils_dir.'/jdiff.pl -r '.join(' ', array_map('escapeshellarg', $to_compare))
            ." --link '$www_this?goto='"); 
    
    exit;
}


$results = get_results();
if ( $results ) krsort($results);

?>
<?/* Page header */?>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en" lang="en">

<head>
    <meta http-equiv="Content-Type" content="text/html; charset=utf-8" />
	<title>Test Results</title>
    <style type="text/css">
        table {
            border: 1px solid black;
            border-collapse: collapse;
        }
        table td, table th {
            border: 1px solid black;
            padding: 8px;
            white-space: nowrap;
        }
        a {
    		color: #00007f;
    		text-decoration: none;
    	}
    	a:hover {
    		color: #00007f;
    		text-decoration: underline;
    	}
    </style>
</head>

<body>
<?/* Page header end */?>

<script type="text/javascript">
// <![CDATA[
function count_checked() {
	var num = 0;
	var form = document.results_list;
	for (var i=0; i<form.length; ++i) {
		if ((form[i].type.toLowerCase() == 'checkbox') && (form[i].name != 'check_all') && form[i].checked) {
			++num;
		}
	}
	return num;
}

function update_state() {
	var button;
	var num_checked = count_checked();
	for (var i = 1; i <= 2; ++i) {
		button = document.getElementById('remove_button' + i);
		if (button) {
			button.disabled = (num_checked == 0);
		}
		button = document.getElementById('compare_button' + i);
		if (button) {
			button.disabled = (num_checked < 2);
		}
	}
}

function confirm_compare() {
	var num = count_checked();
	if (num < 2) {
		alert('At least two results should be selected!');
		return false;
	}
	else
		return true;
}
// ]]>
</script>

<h1>Test Results</h1>

<?
if ( $results ) {
    ?>
    <form method="post" action="?compare" name="results_list" target="_blank">
    <? if ( isset($utils_dir) ) { ?>
    <p><input type="submit" name="compare" id="compare_button1" value="Compare Selected Results" disabled="disabled" onclick="javascript:return confirm_compare();" /></p>
    <? } ?>
    <table>
    <tr>
    <td></td>
    <td><b>Detailed Report</b></td>
    <td><b>Summary</b></td>
    <td><b>Comment</b></td>
    </tr>
    <?
    foreach ( $results as $result ) {
        if ( $result['host'] ) {
            $link_text = '<b>'.html($result['host']).'</b>'
                .' ('.html($result['arch']).')'
                .' '.$result['dt']['h'].':'.$result['dt']['mi'].':'.$result['dt']['s']
                .' '.$result['dt']['d'].'/'.$result['dt']['m'].'/'.$result['dt']['y']
                ;
        } else {
            $link_text = html($result['name']);
        }
        $link_addr = $www_this."?goto=".$result['name'];
        ?>
        <tr><td>
        <? if ( isset($utils_dir) ) { ?>
        <input type='checkbox' name='<?=html($result['name'])?>' onclick='javascript:update_state();'/></td>
        <? } ?>
        <td>
        <a href="<?=html($link_addr)?>" style='font-size:16pt'><?=$link_text?></a>
        </td><td>
        <code><?=$result['summary'];?></code>
        </td><td>
        <code><?=$result['comment'];?></code>
        </td></tr>
        <?
    }
    ?>
    </table>
    <p><input type="submit" name="compare" id="compare_button1" value="Compare Selected Results" disabled="disabled" onclick="javascript:return confirm_compare();" /></p>
    </form>
    <?
}
else {
// Empty dir
    ?><h2>Empty</h2><?
}
?>

<?/* Page footer */?>

</body>
</html>
<?/* Page footer end */?>
