#!/usr/bin/perl -w

# Distribution Checker
# Get Certified Page Module (tests_cert.pl)
#
# Copyright (C) 2007-2009 The Linux Foundation. All rights reserved.
#
# This program has been developed by ISP RAS for LF.
# The ptyshell tool is originally written by Jiri Dluhos <jdluhos@suse.cz>
# Copyright (C) 2005-2007 SuSE Linux Products GmbH
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# version 2 as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

use Templates;
use BuildList;
use UserProfile;

my @error_msgs = ();
my @warn_msgs = ();

if ($_COOKIE{"last_error_text"}) {
	push @error_msgs, $_COOKIE{"last_error_text"};
}

# Create session
if (!-d $SERVER_PARAM{"APP_DATA"}."/profiles") {
	system("mkdir -p ".$SERVER_PARAM{"APP_DATA"}."/profiles");
	if (!-d $SERVER_PARAM{"APP_DATA"}."/profiles") {
		push @error_msgs, "Cannot not create the profiles directory!<br />$!";
	}
}

my $profile_name = "";
my $create_session_error = 0;

if (!$_COOKIE{"session_id"} or !-f $SERVER_PARAM{"APP_DATA"}."/profiles/~session.".$SERVER_PARAM{"PEER_IP"}.".".$_COOKIE{"session_id"}) {
	my $i = time();
	while (-f $SERVER_PARAM{"APP_DATA"}."/profiles/~session.".$SERVER_PARAM{"PEER_IP"}.".$i") {
		$i++;
	}
	$_COOKIE{"session_id"} = $i;
	$profile_name = "~session.".$SERVER_PARAM{"PEER_IP"}.".$i";
	if (open(FILE, ">".$SERVER_PARAM{"APP_DATA"}."/profiles/$profile_name")) {
		close(FILE);
	}
	else {
		push @error_msgs, "Cannot create the session profile '$profile_name'!<br />$!";
		$create_session_error = 1;
	}
}
else {
	$profile_name = "~session.".$SERVER_PARAM{"PEER_IP"}.".".$_COOKIE{"session_id"};
}

my @checkbox_state = ('', ' checked="checked"');
my $profile = read_profile($SERVER_PARAM{"APP_DATA"}."/profiles/$profile_name");
my $standard_ver = $DEFAULT_STANDARD_VER;
if (detect_OS() =~ m/\bMoblin\b/i) {
	$standard_ver = 'Moblin 2.0';
}
my %default_profile_general = (
	"NAME" => "",
	"ORGANIZATION" => "",
	"EMAIL" => "",
	"SEND_EMAIL" => "0",
	"VERBOSE_LEVEL" => "1",
	"ARCHITECTURE" => (detect_architecture() or "x86"),
	"USE_INTERNET" => "1",
	"STD_VERSION" => $standard_ver,
	"STD_PROFILE" => "no",
	"COMMENTS" => ""
);

# Fill values not specified in profile
foreach (keys %default_profile_general) {
	if (!defined($profile->{'GENERAL'}->{$_})) {
		$profile->{'GENERAL'}->{$_} = $default_profile_general{$_};
	}
}

# Set cookie expiration to 7 days since now
my ($sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdst) = gmtime(time() + 7*86400);
my @wday_names = qw/Sunday Monday Tuesday Wednesday Thursday Friday Saturday/;
my @mon_names = qw/Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec/;
# Format date like follows: Tuesday, 31-Dec-19 21:00:00 GMT
my $fmt_date = sprintf("%s, %d-%s-%d %02d:%02d:%02d GMT", $wday_names[$wday], $mday, $mon_names[$mon], 1900 + $year, $hour, $min, $sec);
print "HTTP/1.0 200 OK" . CRLF;
print "Content-type: text/html" . CRLF;
print "Set-Cookie: session_id=".$_COOKIE{"session_id"}."; expires=$fmt_date; path=/;" . CRLF;
print "Set-Cookie: last_error_text=; path=/;" . CRLF . CRLF;

print_header("Get Certified", "get_cert");

my $table_spacer = "<td><img src=\"images/environment-spacer.gif\" width=\"5\" height=\"1\" border=\"0\" alt=\"\" /></td>";

# Show errors/warnings
#push @error_msgs, $profile_error if ($profile_error);      # Ignore incompatible profile format
#@error_msgs = (@error_msgs, @manifest_errors);
@warn_msgs = (@profile_warnings);
my $error_text = "";
if ((scalar(@error_msgs) != 0) or (scalar(@warn_msgs) != 0)) {
	$error_text = "<ul>";
	foreach (@error_msgs) {
		$error_text .= "<li><font color=\"red\"><u><b>Error:</b></u></font> <font color=\"darkblue\">$_</font></li>";
	}
	foreach (@warn_msgs) {
		$error_text .= "<li><font color=\"darkred\"><u><b>Warning:</b></u></font> <font color=\"darkblue\">$_</font></li>";
	}
	$error_text .= "</ul>";
}

print show_error_dlg($error_text);

my %field_contents = ();
$field_contents{'NAME'} = $default_profile_general{'NAME'};
$field_contents{'ORGANIZATION'} = $default_profile_general{'ORGANIZATION'};
$field_contents{'COMMENTS'} = $default_profile_general{'COMMENTS'};
$field_contents{'EMAIL'} = $default_profile_general{'EMAIL'};
for my $key (keys %field_contents) {
	$field_contents{$key} =~ s/&/&amp;/g;
	$field_contents{$key} =~ s/\"/&quot;/g;
}

print <<DATA;

<div id="tooltip" style="display: none; background-color: #ffffd0; border: solid 1px black; position: absolute; z-index: 1000;">
<table cellpadding="0" cellspacing="0" border="0" style="width: 17em;">
  <tr style="background-color: #d0d0d0; color: black;">
    <td align="right" style="border-bottom: solid 1px black;"><img src="images/environment-spacer.gif" width="7" height="1" alt="" /></td>
    <td align="center" width="100%" style="border-bottom: solid 1px black;"><b>Information</b></td>
    <td align="right" style="border-bottom: solid 1px black;"><img src="images/close.png" onclick="javascript:initDialog(null);" width="20" height="20" title="Close" alt="X" style="cursor: pointer;" /></td>
  </tr>
  <tr>
    <td colspan="3" style="padding: 5px; white-space: nowrap;" id="tooltip_text"></td>
  </tr>
</table>
</div>

<div id="ajax_loading" style="display: none; position: fixed; width: 13em; height: 5.8em; top: 40%; left: 50%; margin-top: -2.9em; margin-left: -7.5em; border: solid 1px #00007f; background-color: #e7f0ff; text-align: center; padding: 6px;"><div align="center">
Please, wait...<br />
<img src="images/environment-spacer.gif" width="1" height="5" alt="" /><br />
<img src="images/ajax_progress.gif" width="16" height="16" alt="" /><br />
<img src="images/environment-spacer.gif" width="1" height="7" alt="" /><br />
<input type="button" name="ajax_abort" id="ajax_abort" value="Abort" style="border: solid 1px #00007f; background: #e0e0ff; color: blue; width: 6em;" onclick="javascript:onAbort();" />
</div></div>

<table cellpadding="0" cellspacing="0" border="0" style="display: none; position: absolute; z-index: 999;" id="shadow">
  <tr>
    <td width="14" height="14" style="background: url('images/shadow_tl.png') no-repeat;"></td>
    <td height="14" style="background: url('images/shadow_tc.png') repeat-x;"></td>
    <td width="14" height="14" style="background: url('images/shadow_tr.png') no-repeat;"></td>
  </tr>
  <tr>
    <td width="14" style="background: url('images/shadow_ml.png') repeat-y;"></td>
    <td style="background: url('images/shadow_mc.png') repeat;" id="shadow_center"></td>
    <td width="14" style="background: url('images/shadow_mr.png') repeat-y;"></td>
  </tr>
  <tr>
    <td width="14" height="14" style="background: url('images/shadow_bl.png') no-repeat;"></td>
    <td height="14" style="background: url('images/shadow_bc.png') repeat-x;"></td>
    <td width="14" height="14" style="background: url('images/shadow_br.png') no-repeat;"></td>
  </tr>
</table>

<h1>Configure Tests</h1>
<form method="post" action="tests_conf_save.pl" name="configure_test" id="configure_test">
<input type="hidden" name="session_id" id="session_id" value="$_COOKIE{session_id}" />
<input type="hidden" name="form_action" id="form_action" value="" />
<input type="hidden" name="use_internet" id="use_internet" value="1" />
<input type="hidden" name="certify" id="certify" value="1" />
<table border="0" cellpadding="0" cellspacing="0">
  <tr>
    <td valign="top" width="50%">
      <table border="0" cellpadding="2" cellspacing="2" width="100%">
        <tr>
          <td>Your&nbsp;name:&nbsp;<font color="red">*</font></td>
          $table_spacer
          <td colspan="2"><input type="text" name="tester_name" id="tester_name" value="$field_contents{NAME}" style="width: 13em;" onfocus="javascript:style.borderColor=input_border_color;" /></td>
        </tr>
        <tr>
          <td>Organization:&nbsp;<font color="red">*</font></td>
          $table_spacer
          <td colspan="2"><input type="text" name="organization" id="organization" value="$field_contents{ORGANIZATION}" style="width: 13em;" onfocus="javascript:style.borderColor=input_border_color;" /></td>
        </tr>
        <tr>
          <td>E-mail:</td>
          $table_spacer
          <td><input type="text" name="tester_email" id="tester_email" value="$field_contents{EMAIL}" style="width: 13em;" /></td>
          <td width="100%" style="white-space: nowrap;"><label><input type="checkbox" name="send_email_report" id="send_email_report"$checkbox_state[$default_profile_general{SEND_EMAIL}] /> Send report via E-mail</label></td>
        </tr>
        <tr>
          <td colspan="4" style="border: dashed 1px gray; background: #ffc000; cursor: pointer;" onclick="javascript:show_hide_details();">&nbsp;<img src="images/arr_d.png" alt="" id="block_arrow_img" width="12" height="12" />&nbsp;&nbsp;Additional options</td>
        </tr>
        <tr>
          <td colspan="4" id="additional_data" style="border: dashed 1px gray; border-top: none; padding-right: 6px;">
            <table border="0" cellpadding="2" cellspacing="2" width="100%">
              <tr>
                <td>Verbose&nbsp;Level:</td>
                $table_spacer
                <td width="100%">
                  <select name="verbose_level" id="verbose_level" style="width: 13em;">
DATA
print "                    <option value=\"1\"".(($default_profile_general{'VERBOSE_LEVEL'} == 1) ? " selected=\"selected\"" : "").">1 (normal)</option>\n";
print "                    <option value=\"2\"".(($default_profile_general{'VERBOSE_LEVEL'} == 2) ? " selected=\"selected\"" : "").">2 (high)</option>\n";
print <<DATA;
                  </select>
                </td>
              </tr>
              <tr>
                <td colspan="3">Comments:</td>
              </tr>
              <tr>
                <td colspan="3">
                  <textarea name="comments" id="comments" rows="5" cols="68" style="width: 100%;">$field_contents{COMMENTS}</textarea>
                </td>
              </tr>
            </table>
          </td>
        </tr>
        <tr>
          <td colspan="4" valign="bottom" height="15"><hr /></td>
        </tr>
        <tr>
          <td valign="middle" colspan="4">
            <table border="0" cellpadding="0" cellspacing="0" width="100%">
              <tr>
                <td>Architecture:</td>
                $table_spacer
                <td>
                  <select name="architecture" id="architecture" style="width: 6em;">
DATA

if (scalar(@supported_archs) == 0) {
	print "                    <option value=\"".$default_profile_general{'ARCHITECTURE'}."\" selected=\"selected\">".$default_profile_general{'ARCHITECTURE'}."</option>\n";
}
else {
	foreach (@supported_archs) {
		if ($_ eq $default_profile_general{'ARCHITECTURE'}) {
			print "                    <option value=\"$_\" selected=\"selected\">$_</option>\n";
		}
		else {
			print "                    <option value=\"$_\">$_</option>\n";
		}
	}
}

print <<DATA;
                  </select>
                </td>
                <td><img src="images/environment-spacer.gif" width="10" height="1" border="0" alt="" /></td>
                <td>Standard:</td>
                $table_spacer
                <td>
                  <select name="std_version" id="std_version" style="width: 9em;" onchange="javascript:control_profile_list();">
DATA

my $group = '';
foreach my $stdv (@all_std_versions) {
	my $curgroup = 'Unknown';
	if ($stdv =~ m/^(\S+) .*$/) {
		$curgroup = $1;
	}
	if ($group ne $curgroup) {
		if ($group ne '') {
			print "                    </optgroup>\n";
		}
		print "                    <optgroup label=\"$curgroup\">\n";
		$group = $curgroup;
	}
	if ($stdv eq $default_profile_general{'STD_VERSION'}) {
		print "                      <option value=\"$stdv\" selected=\"selected\">$stdv</option>\n";
	}
	else {
		print "                      <option value=\"$stdv\">$stdv</option>\n";
	}
}
if ($group ne '') {
	print "                    </optgroup>\n";
}

print <<DATA;
                  </select>
                </td>
                <td><img src="images/environment-spacer.gif" width="10" height="1" border="0" alt="" /></td>
                <td>Profile:</td>
                <td><img src="images/environment-spacer.gif" width="5" height="1" border="0" alt="" /></td>
                <td width="100%">
                  <select name="std_profile" id="std_profile" style="width: 16em;">
                    <option value="no" selected="selected">Empty</option>
                  </select>
                </td>
              </tr>
            </table>
          </td>
        </tr>
      </table>
      <table cellpadding="0" cellspacing="0" width="100%" style="border: solid 4px white;"><tr><td>
        &nbsp;<b><span class="active" onclick="javascript:var e=document.getElementById('warning');if (e) {e.style.display=(e.style.display=='none'?'':'none');}" title="Click here for details"><font color="red">WARNING!</font> <font color="green">Do not use the tests on any machine holding important data!</font></span></b><br />
        <div id="warning" style="display: none;"><br />
        <div style="border: 1px dashed #2f6fab; background-color: #f9f9f9; width: 90%; padding: 3px; font-family: monospace;">
          WARNING! PLEASE DO NOT USE THIS SCRIPT ON ANY MACHINE THAT HOLDS IMPORTANT DATA! The tests sometimes put a heavy stress on the tested computer, and in rare unfortunate cases they can crash the system or damage the stored data. They also are a security risk as they pass along the root password.
        </div><br />
        </div>
      </td></tr></table>
      <table cellpadding="0" cellspacing="0" width="100%" style="border: solid 4px white;"><tr><td>
        <input type="button" name="run_tests" id="run_tests_button" value="Run the Tests" style="width: 11em;" onclick="javascript:do_action('run_tests');" /><!--&nbsp;&nbsp; (Estimated&nbsp;execution&nbsp;time:&nbsp;<b><span id="estim_time">0&nbsp;min</span></b>)-->
      </td></tr></table>
    </td>
    <td style="border-right: dashed 1px gray;"><img src="images/environment-spacer.gif" width="15" height="1" border="0" alt="" /></td>
    <td><img src="images/environment-spacer.gif" width="15" height="1" border="0" alt="" /></td>
DATA

my $profiles_options = "";
my @profiles_list;
if (opendir(DIR, $SERVER_PARAM{"APP_DATA"}."/profiles")) {
	@profiles_list = sort grep !/^[~\.]/, readdir(DIR);
	foreach (@profiles_list) {
		$profiles_options .= "                <option value=\"$_\">$_</option>\n";
	}
	closedir(DIR);
}

my $disabled_list = "";
if ($profiles_options eq "") {
	$profiles_options = "                <option value=\"\">&lt;No profiles present&gt;</option>\n";
	$disabled_list = " disabled=\"disabled\"";
}

my $profiles_list_num = scalar(@profiles_list);
if ($profiles_list_num < 2) {
	$profiles_list_num = 2;
}
elsif ($profiles_list_num > 6) {
	$profiles_list_num = 6;
}

my $current_std = $profile->{'GENERAL'}->{'STD_VERSION'};
$current_std =~ s/ /:/g;
my $current_arch = $profile->{'GENERAL'}->{'ARCHITECTURE'};

print <<DATA;
    <td valign="top" width="50%">
      <h2>User Profile Management&nbsp;&nbsp;&nbsp;<b><span class="active" onclick="javascript:show_hide_tooltip(this, 'profile_management', event);"><img src="images/question.png" width="16" height="17" alt="[?]" title="What's this?" /></span></b></h2>
      <table border="0" cellpadding="2" cellspacing="2" width="100%">
        <tr>
          <td colspan="3">Select the profile or type its name:</td>
        </tr>
        <tr>
          <td colspan="3">
            <table border="0" cellpadding="0" cellspacing="0"><tr>
              <td style="vertical-align: middle;">
                <input type="text" name="edit_profile_name" id="edit_profile_name" value="" style="width: 18em;" onkeypress="javascript:return onKeyPressEdit(event);" onfocus="javascript:style.borderColor=input_border_color;" />
              </td>
              <td style="vertical-align: middle;">
                <img src="images/drop_down.png" alt="v" width="16" height="20" id="drop_down_button" style="cursor: pointer;" onclick="javascript:onClickArrow();" />
              </td>
            </tr></table>
            <div id="hidden_div" style="position: absolute; z-index: +1; display: none;">
              <select name="list_profile_name" id="list_profile_name" size="$profiles_list_num" style="font-family: monospace; font-size: 100%; width: 19.7em;"$disabled_list onclick="javascript:onClickList();" onkeypress="javascript:return onKeyPressList(event);">
$profiles_options              </select>
            </div>
          </td>
        </tr>
        <tr>
          <td><input type="button" name="save_profile_button" id="save_profile_button" value="Save" style="width: 5.8em;" onclick="javascript:onSave();" /></td>
          <td><input type="button" name="load_profile_button" id="load_profile_button" value="Load" style="width: 5.8em;" onclick="javascript:onLoad();" /></td>
          <td width="100%"><input type="button" name="delete_profile_button" id="delete_profile_button" value="Delete" style="width: 5.8em;" onclick="javascript:onDelete();" /></td>
        </tr>
      </table>
    </td>
  </tr>
</table>
</form>

<script language="javascript" type="text/javascript">
// <![CDATA[
// Preload images for shadow, various icons, buttons, etc.
preload_images(new Array('arr_r.png', 'expand.png', 'collapse.png', 'drop_down_pressed.png', 'close.png', 'shadow_bc.png', 'shadow_bl.png', 'shadow_br.png', 'shadow_mc.png', 'shadow_ml.png', 'shadow_mr.png', 'shadow_tc.png', 'shadow_tl.png', 'shadow_tr.png'));

// Global arrays and scalars dynamically redefined via AJAX:
var current_arch_std;   // String containing current architecture and standard version.
var arch_list;          // List of supported architectures.
var stdv_list;          // List of supported standard versions.
var stdp_list;          // List of profiles present in each standard version.
var stdp_def_list;      // Default profiles.
var profiles_list;      // List of user profiles.

// First initialization of variables used later in AJAX
function init_data() {
	current_arch_std = '$current_arch-$current_std';
DATA

print "\tprofiles_list = new Array(".join(', ', map("'$_'", @profiles_list)).");\n";

print generate_std_profiles();

print <<DATA;
}

////////////////////////////////////////////////////////////////////////////////
// Graphical Interface

// Find and cache most of the necessary HTML document elements.
var edit_profile_name = document.getElementById('edit_profile_name');
var list_profile_name = document.getElementById('list_profile_name');
var hidden_div = document.getElementById('hidden_div');
var drop_down_button = document.getElementById('drop_down_button');
var main_form = document.getElementById('configure_test');
var dialog_shadow = document.getElementById('shadow');
var dialog_shadow_center = document.getElementById('shadow_center');
if (!list_profile_name || !edit_profile_name || !hidden_div || !drop_down_button || !main_form || !dialog_shadow || !dialog_shadow_center)
	alert('Internal error: Cannot find essential HTML elements! JavaScript will not work correctly.\\nPlease, contact the authors at linux\@ispras.ru.');

var input_border_color = ((navigator.appName == 'Konqueror') ? '#cccccc' : '');

var just_shown_dialog = false;          // Indicates that the dialog was just shown (to prevent its hiding from onBodyClick)
var dialog = null;                      // The dialog itself
var dialog_caller = null;               // Element which caused the dialog to appear
var dialog_caller_class = null;         // Normal class name of the dialog caller (to be restored when the dialog is closed)

// Determines absolute dynamic position of an element (1 pixel below its left bottom corner).
// Used for displaying the dialog box just below the element.
function getPosition(elem) {
	if (!elem)
		return {x: -1, y: -1};
	var x = 0;
	var y = elem.offsetHeight;
	while (elem) {
		x += elem.offsetLeft;
		y += elem.offsetTop;
		elem = elem.offsetParent;
	}
	return {x: x, y: y};
}

// Hides the previous dialog (if it was shown) and prepare data for
// displaying the new one.
// CALLER - the HTML element which caused the dialog to appear and (possibly)
// will change its appearance because of that.
function initDialog(caller) {
	if (dialog_caller && dialog_caller_class)
		dialog_caller.className = dialog_caller_class;
	dialog_shadow.style.display = 'none';
	if (dialog)
		dialog.style.display = 'none';
	if (!caller || (caller == dialog_caller)) {
		dialog_caller = null;
		dialog_caller_class = null;
		return false;
	}
	else {
		dialog_caller = caller;
		dialog_caller_class = caller.className;
		return true;
	}
}

// Shows the dialog at the specified position and adjusts the shadow below it.
function showDialog(show_dlg, pos) {
	dialog = show_dlg;
	dialog.style.left = pos.x + 'px';
	dialog.style.top = pos.y + 'px';
	dialog.style.display = '';
	just_shown_dialog = true;
	dialog_shadow_center.width = dialog.offsetWidth - 15;
	dialog_shadow_center.height = dialog.offsetHeight - 15;
	dialog_shadow.style.left = pos.x + 'px';
	dialog_shadow.style.top = pos.y + 'px';
	dialog_shadow.style.display = '';
}

// Profile management functions
function control_profile_list() {
	var elem_v = document.getElementById('std_version');
	var elem_p = document.getElementById('std_profile');
	if (!elem_v || !elem_p)
		return;
	var current_std = elem_v.value.replace(/ /g, ':');
	var current_profile = elem_p.value;
	if (current_profile == 'no')
		current_profile = '$default_profile_general{'STD_PROFILE'}';
	var profile_list_html = '';
	var count = 0;

	var elem_parent = elem_p.parentNode;
	elem_parent.removeChild(elem_p);
	elem_p = document.createElement('select');
	elem_p.id = 'std_profile';
	elem_p.name = 'std_profile';
	elem_p.style.width = '16em';
	elem_parent.appendChild(elem_p);

	var new_profile = stdp_def_list[current_std];
	for (var id in stdp_list[current_std]) {
		++count;
		var tmp_node = document.createElement('option');
		tmp_node.setAttribute('value', id);
		tmp_node.appendChild(document.createTextNode(stdp_list[current_std][id]));
		elem_p.appendChild(tmp_node);
		// Try to keep the currently selected profile; take the first entry if failed.
		if (!new_profile && (count == 1))
			new_profile = id;
		if (id == current_profile)
			new_profile = current_profile;
	}
	if (count == 0) {
		var tmp_node = document.createElement('option');
		tmp_node.setAttribute('value', 'no');
		tmp_node.appendChild(document.createTextNode('<No profiles>'));
		elem_p.appendChild(tmp_node);
		new_profile = 'no';
	}
	elem_p.disabled = (count <= 1);
	elem_p.value = new_profile;
}

////////////////////////////////////////////////////////////////////////////////
//  Drop-down List for User Profiles

var VK_ESC  = 27;
var VK_CR   = 13;
var VK_LF   = 10;
var VK_DOWN = 40;
var VK_TAB  = 9;

var just_shown_list = false;

function show_list() {
	hidden_div.style.display = '';
	drop_down_button.setAttribute('src', 'images/drop_down_pressed.png');
	just_shown_list = true;
}

function hide_list(set_focus) {
	if (set_focus)
		edit_profile_name.focus();
	hidden_div.style.display = 'none';
	drop_down_button.setAttribute('src', 'images/drop_down.png');
}

function onBodyClick() {
	if (hidden_div.style.display != 'none') {
		if (!just_shown_list)
			hide_list(false);
		just_shown_list = false;
	}
	if (!just_shown_dialog)
		initDialog(null);
	just_shown_dialog = false;
}

function onKeyPressList(event) {
	if (hidden_div.style.display == 'none')
		return true;
	if (event.keyCode == VK_ESC) {
		hide_list(true);
		return false;
	}
	else if ((event.keyCode == VK_CR) || (event.keyCode == VK_LF)) {
		edit_profile_name.value = list_profile_name.value;
		hide_list(true);
		return false;
	}
	else if (event.keyCode == VK_TAB) {
		hide_list(false)
		return true;
	}
	return true;
}

function onClickList() {
	if (hidden_div.style.display == 'none')
		return true;
	edit_profile_name.value = list_profile_name.value;
	hide_list(true);
}

function onKeyPressEdit(event) {
	if (event.keyCode == VK_ESC) {
		if (hidden_div.style.display != 'none') {
			hide_list(true);
			return false;
		}
	}
	else if (event.keyCode == VK_DOWN) {
		if (hidden_div.style.display == 'none') {
			show_list();
		}
		list_profile_name.focus();
		return false;
	}
	else if (event.keyCode == VK_TAB) {
		if (hidden_div.style.display != 'none') {
			list_profile_name.focus();
			return false;
		}
	}
	else
		hide_list(true);
	return true;
}

function onClickArrow() {
	if (hidden_div.style.display == 'none') {
		show_list();
		list_profile_name.focus();
	}
	else {
		hide_list(true);
	}
}

////////////////////////////////////////////////////////////////////////////////
// Tooltips

var tooltip_texts = new Array();
tooltip_texts['profile_management'] = 'This section allows you to save all the settings<br />from the left pane into user profiles and load them afterwards.';

function show_hide_tooltip(elem_parent, id, event) {
	if (!initDialog(elem_parent))
		return;
	var tt_window = document.getElementById('tooltip');
	if (tt_window) {
		var tt_text = document.getElementById('tooltip_text');
		if (tt_text) {
			if (tooltip_texts[id])
				tt_text.innerHTML = tooltip_texts[id] + '<br /><div align="right"><a href="tests_help.pl#' + id + '" target="_blank" onclick="javascript:initDialog(null);"><i>Details...</i></a></div>';
			else
				tt_text.innerHTML = '<font color="red"><b>Sorry, no information for this item was found!</b></font><br />Please, write to <a href="mailto:linux\@ispras.ru">linux\@ispras.ru</a>.';
			var scrollTop = document.body.scrollTop;
			if (scrollTop == 0)
			{
				if (window.pageYOffset)
					scrollTop = window.pageYOffset;
				else
					scrollTop = (document.body.parentElement) ? document.body.parentElement.scrollTop : 0;
			}
			var scrollLeft = document.body.scrollLeft;
			if (scrollLeft == 0)
			{
				if (window.pageXOffset)
					scrollLeft = window.pageXOffset;
				else
					scrollLeft = (document.body.parentElement) ? document.body.parentElement.scrollLeft : 0;
			}
			var div_top = event.clientY + scrollTop + 10;
			var div_left = event.clientX + scrollLeft + 10;
			tt_window.style.display = '';
			var div_width = (tt_window.style.pixelWidth || tt_window.offsetWidth);
			var wnd_width = (document.body.clientWidth || window.innerWidth);
			if (div_left + div_width > wnd_width + scrollLeft)
				div_left = wnd_width + scrollLeft - div_width;
			if (div_left < scrollLeft)
				div_left = scrollLeft;
			showDialog(tt_window, {x: div_left, y: div_top});
		}
	}
}

////////////////////////////////////////////////////////////////////////////////
// User Profiles

function set_options_by_profile() {
DATA

print generate_profile_init($profile, 'cert');

print <<DATA;
}

////////////////////////////////////////////////////////////////////////////////
// Main Configuring and Testing Functionality

// Shows/hides the block with additional options
function show_hide_details() {
	var elem_div = document.getElementById('additional_data');
	var elem_img = document.getElementById('block_arrow_img');
	if (elem_div && elem_img) {
		var to_show = (elem_div.style.display == 'none');
		elem_div.style.display = to_show ? '' : 'none';
		elem_img.setAttribute('src', to_show ? 'images/arr_d.png' : 'images/arr_r.png');
	}
}

// Performs pre-testing checks
function do_action(name) {
	var elem = document.getElementById('form_action');
	if (!elem) {
		alert("Internal error: Cannot find 'form_action' field!\\nPlease, contact the authors.");
		return false;
	}
	else if (name == 'run_tests') {
		var err = false;
		var elem1 = document.getElementById('organization');
		var elem2 = document.getElementById('tester_name');
		if (!elem1 || (elem1.value == '')) {
			elem1.style.borderColor = 'red';
			err = true;
		}
		if (!elem2 || (elem2.value == '')) {
			elem2.style.borderColor = 'red';
			err = true;
		}
		if (err) {
			alert('Please, specify your name and organization!');
			return false;
		}
	}
	elem.value = name;
	if (name == 'run_tests') {
		ajax_call_post('action=check_run', 'configure_test');
	}
	return true;
}

////////////////////////////////////////////////////////////////////////////////
// Special Functions

function save_and_go(event) {
	var url = this.getAttribute('href');
	// IE fix
	if (!event)
		event = window.event;
	document.configure_test.target = (event.shiftKey ? '_blank' : '');
	document.configure_test.action = 'tests_conf_save.pl?goto=' + url.replace('#', '%23');
	document.configure_test.submit();
	document.configure_test.action = 'tests_conf_save.pl';
	document.configure_test.target = '';
	event.cancelBubble = true;
	return false;
}

function modify_all_links() {
	var links = document.getElementsByTagName('A');
	for (var i = 0; i < links.length; i++) {
		if ((links[i].target == '') && (links[i].href != '') && (links[i].onclick == null)) {
			links[i].onclick = save_and_go;
		}
	}
}

////////////////////////////////////////////////////////////////////////////////
//  AJAX

function onSave() {
	if (edit_profile_name.value == '') {
		edit_profile_name.style.borderColor = 'red';
		alert('Please, specify the profile name!');
	}
	else if (edit_profile_name.value.search(/[\\/\\\\<>|\\?\\*]/) != -1) {
		edit_profile_name.style.borderColor = 'red';
		alert('The profile name should contain only symbols that are allowed in file names!');
	}
	else {
		for (var i=0; i<profiles_list.length; ++i) {
			if (edit_profile_name.value == profiles_list[i]) {
				if (!confirm("Do you want to overwrite the profile '" + edit_profile_name.value + "'?"))
					return;
				break;
			}
		}
		ajax_call_post('action=save_profile', 'configure_test');
	}
}

function onLoad() {
	if (edit_profile_name.value) {
		ajax_call_get('action=load_profile&page=cert&profile=' + encodeURIComponent(edit_profile_name.value));
	}
	else {
		edit_profile_name.style.borderColor = 'red';
		alert('Please, specify the profile name!');
	}
}

function onDelete() {
	if (edit_profile_name.value) {
		if (confirm("Do you really want to delete the profile '" + edit_profile_name.value + "'?"))
			ajax_call_get('action=delete_profile&profile=' + encodeURIComponent(edit_profile_name.value));
	}
	else {
		edit_profile_name.style.borderColor = 'red';
		alert('Please, specify the profile name!');
	}
}

function ajaxProcessResult(responseXML) {
	edit_profile_name.value = '';
	if (responseXML.getElementsByTagName('profiles').length > 0) {
		var elem_list = document.getElementById('list_profile_name');
		var elem_div = elem_list.parentNode;
		elem_div.removeChild(elem_list);
		elem_list = document.createElement('select');
		elem_list.id = 'list_profile_name';
		elem_list.name = 'list_profile_name';
		elem_list.style.fontFamily = 'monospace';
		elem_list.style.fontSize = '100%';
		elem_list.style.width = '19.7em';
		elem_list.onclick = onClickList;
		elem_list.onkeypress = onKeyPressList;
		elem_div.appendChild(elem_list);
		list_profile_name = elem_list;
		profiles_list = new Array();

		var profile_names = responseXML.getElementsByTagName('profiles')[0].getElementsByTagName('name');
		if (profile_names.length > 0) {
			for (var i=0; i<profile_names.length; ++i) {
				profiles_list[i] = profile_names[i].childNodes[0].nodeValue;
				var tmp_node = document.createElement('option');
				tmp_node.setAttribute('value', profiles_list[i]);
				tmp_node.appendChild(document.createTextNode(profiles_list[i]));
				elem_list.appendChild(tmp_node);
			}
			elem_list.disabled = false;
			if (profile_names.length < 2)
				elem_list.size = 2;
			else if (profile_names.length > 6)
				elem_list.size = 6;
			else
				elem_list.size = profile_names.length;
		}
		else {
			var tmp_node = document.createElement('option');
			tmp_node.setAttribute('value', '');
			tmp_node.appendChild(document.createTextNode('<No profiles present>'));
			elem_list.appendChild(tmp_node);
			elem_list.disabled = true;
			elem_list.size = 2;
		}
	}
	if (responseXML.getElementsByTagName('js').length > 0) {
		var js_data = '';
		for (var i=0; i<responseXML.getElementsByTagName('js').length; ++i) {
			for (var j=0; j<responseXML.getElementsByTagName('js')[i].childNodes.length; ++j)
				js_data += responseXML.getElementsByTagName('js')[i].childNodes[j].nodeValue;
		}
		eval(js_data);
	}
	if (responseXML.getElementsByTagName('testrun_allowed').length > 0) {
		var testrun_allowed = responseXML.getElementsByTagName('testrun_allowed')[0];
		if (testrun_allowed)
			document.configure_test.submit();
	}
}

////////////////////////////////////////////////////////////////////////////////
// Initialization

init_data();
modify_all_links();
set_options_by_profile();
// ]]>
</script>
DATA

print_footer();
