#!/usr/bin/perl -w

# Distribution Checker
# Custom Tests Page Module (tests_conf.pl)
#
# Copyright (C) 2007-2009 The Linux Foundation. All rights reserved.
#
# This program has been developed by ISP RAS for LF.
# The ptyshell tool is originally written by Jiri Dluhos <jdluhos@suse.cz>
# Copyright (C) 2005-2007 SuSE Linux Products GmbH
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# version 2 as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

use Templates;
use BuildList;
use UserProfile;

my @error_msgs = ();
my @warn_msgs = ();

if ($_COOKIE{"last_error_text"}) {
	push @error_msgs, $_COOKIE{"last_error_text"};
}

# Create session
if (!-d $SERVER_PARAM{"APP_DATA"}."/profiles") {
	system("mkdir -p ".$SERVER_PARAM{"APP_DATA"}."/profiles");
	if (!-d $SERVER_PARAM{"APP_DATA"}."/profiles") {
		push @error_msgs, "Cannot not create the profiles directory!<br />$!";
	}
}

my $profile_name = "";
my $create_session_error = 0;

if (!$_COOKIE{"session_id"} or !-f $SERVER_PARAM{"APP_DATA"}."/profiles/~session.".$SERVER_PARAM{"PEER_IP"}.".".$_COOKIE{"session_id"}) {
	my $i = time();
	while (-f $SERVER_PARAM{"APP_DATA"}."/profiles/~session.".$SERVER_PARAM{"PEER_IP"}.".$i") {
		$i++;
	}
	$_COOKIE{"session_id"} = $i;
	$profile_name = "~session.".$SERVER_PARAM{"PEER_IP"}.".$i";
	if (open(FILE, ">".$SERVER_PARAM{"APP_DATA"}."/profiles/$profile_name")) {
		close(FILE);
	}
	else {
		push @error_msgs, "Cannot create the session profile '$profile_name'!<br />$!";
		$create_session_error = 1;
	}
}
else {
	$profile_name = "~session.".$SERVER_PARAM{"PEER_IP"}.".".$_COOKIE{"session_id"};
}

my @checkbox_state = ('', ' checked="checked"');
my $profile = read_profile($SERVER_PARAM{"APP_DATA"}."/profiles/$profile_name");
my $standard_ver = $DEFAULT_STANDARD_VER;
if (detect_OS() =~ m/\bMoblin\b/i) {
	$standard_ver = 'Moblin 2.0';
}
my %default_profile_general = (
	"NAME" => "",
	"ORGANIZATION" => "",
	"EMAIL" => "",
	"SEND_EMAIL" => "0",
	"VERBOSE_LEVEL" => "1",
	"ARCHITECTURE" => (detect_architecture() or "x86"),
	"USE_INTERNET" => "1",
	"STD_VERSION" => $standard_ver,
	"STD_PROFILE" => "no",
	"COMMENTS" => ""
);

# Fill values not specified in profile
foreach (keys %default_profile_general) {
	if (!defined($profile->{'GENERAL'}->{$_})) {
		$profile->{'GENERAL'}->{$_} = $default_profile_general{$_};
	}
}

# Set cookie expiration to 7 days since now
my ($sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdst) = gmtime(time() + 7*86400);
my @wday_names = qw/Sunday Monday Tuesday Wednesday Thursday Friday Saturday/;
my @mon_names = qw/Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec/;
# Format date like follows: Tuesday, 31-Dec-19 21:00:00 GMT
my $fmt_date = sprintf("%s, %d-%s-%d %02d:%02d:%02d GMT", $wday_names[$wday], $mday, $mon_names[$mon], 1900 + $year, $hour, $min, $sec);
print "HTTP/1.0 200 OK" . CRLF;
print "Content-type: text/html" . CRLF;
print "Set-Cookie: session_id=".$_COOKIE{"session_id"}."; expires=$fmt_date; path=/;" . CRLF;
print "Set-Cookie: last_error_text=; path=/;" . CRLF . CRLF;

print_header("Configure Custom Tests", "conf");

my $table_spacer = "<td><img src=\"images/environment-spacer.gif\" width=\"5\" height=\"1\" border=\"0\" alt=\"\" /></td>";

# Show errors/warnings
#push @error_msgs, $profile_error if ($profile_error);      # Ignore incompatible profile format
#@warn_msgs = (@profile_warnings);
my $error_text = "";
if ((scalar(@error_msgs) != 0) or (scalar(@warn_msgs) != 0)) {
	$error_text = "<ul>";
	foreach (@error_msgs) {
		$error_text .= "<li><font color=\"red\"><u><b>Error:</b></u></font> <font color=\"darkblue\">$_</font></li>";
	}
	foreach (@warn_msgs) {
		$error_text .= "<li><font color=\"darkred\"><u><b>Warning:</b></u></font> <font color=\"darkblue\">$_</font></li>";
	}
	$error_text .= "</ul>";
}

print show_error_dlg($error_text);

my %field_contents = ();
$field_contents{'NAME'} = $default_profile_general{'NAME'};
$field_contents{'ORGANIZATION'} = $default_profile_general{'ORGANIZATION'};
$field_contents{'COMMENTS'} = $default_profile_general{'COMMENTS'};
$field_contents{'EMAIL'} = $default_profile_general{'EMAIL'};
for my $key (keys %field_contents) {
	$field_contents{$key} =~ s/&/&amp;/g;
	$field_contents{$key} =~ s/\"/&quot;/g;
}

print <<DATA;

<div id="tooltip" style="display: none; background-color: #ffffd0; border: solid 1px black; position: absolute; z-index: 1000;">
<table cellpadding="0" cellspacing="0" border="0" style="width: 17em;">
  <tr style="background-color: #d0d0d0; color: black;">
    <td align="right" style="border-bottom: solid 1px black;"><img src="images/environment-spacer.gif" width="7" height="1" alt="" /></td>
    <td align="center" width="100%" style="border-bottom: solid 1px black;"><b>Information</b></td>
    <td align="right" style="border-bottom: solid 1px black;"><img src="images/close.png" onclick="javascript:initDialog(null);" width="20" height="20" title="Close" alt="X" style="cursor: pointer;" /></td>
  </tr>
  <tr>
    <td colspan="3" style="padding: 5px; white-space: nowrap;" id="tooltip_text"></td>
  </tr>
</table>
</div>

<div id="ajax_loading" style="display: none; position: fixed; width: 13em; height: 5.8em; top: 40%; left: 50%; margin-top: -2.9em; margin-left: -7.5em; border: solid 1px #00007f; background-color: #e7f0ff; text-align: center; padding: 6px;"><div align="center">
Please, wait...<br />
<img src="images/environment-spacer.gif" width="1" height="5" alt="" /><br />
<img src="images/ajax_progress.gif" width="16" height="16" alt="" /><br />
<img src="images/environment-spacer.gif" width="1" height="7" alt="" /><br />
<input type="button" name="ajax_abort" id="ajax_abort" value="Abort" style="border: solid 1px #00007f; background: #e0e0ff; color: blue; width: 6em;" onclick="javascript:onAbort();" />
</div></div>

<div id="select_version" style="display: none; padding: 7px; background-color: #eeeeee; border: solid 1px #4040cc; position: absolute; z-index: 1000;" onclick="javascript:event.cancelBubble=true;">
</div>

<table cellpadding="0" cellspacing="0" border="0" style="display: none; position: absolute; z-index: 999;" id="shadow">
  <tr>
    <td width="14" height="14" style="background: url('images/shadow_tl.png') no-repeat;"></td>
    <td height="14" style="background: url('images/shadow_tc.png') repeat-x;"></td>
    <td width="14" height="14" style="background: url('images/shadow_tr.png') no-repeat;"></td>
  </tr>
  <tr>
    <td width="14" style="background: url('images/shadow_ml.png') repeat-y;"></td>
    <td style="background: url('images/shadow_mc.png') repeat;" id="shadow_center"></td>
    <td width="14" style="background: url('images/shadow_mr.png') repeat-y;"></td>
  </tr>
  <tr>
    <td width="14" height="14" style="background: url('images/shadow_bl.png') no-repeat;"></td>
    <td height="14" style="background: url('images/shadow_bc.png') repeat-x;"></td>
    <td width="14" height="14" style="background: url('images/shadow_br.png') no-repeat;"></td>
  </tr>
</table>

<h1>Configure Custom Tests</h1>
<form method="post" action="tests_conf_save.pl" name="configure_test" id="configure_test">
<input type="hidden" name="session_id" id="session_id" value="$_COOKIE{session_id}" />
<input type="hidden" name="form_action" id="form_action" value="" />
<input type="hidden" name="std_profile" id="std_profile" value="$default_profile_general{STD_PROFILE}" />
<input type="hidden" name="file_inst" id="file_inst" value="" />
<input type="hidden" name="file_dl" id="file_dl" value="" />
<input type="hidden" name="ts_glob_options" id="ts_glob_options" value="" />
<input type="hidden" name="auto_replies" id="auto_replies" value="" />
<table border="0" cellpadding="0" cellspacing="0">
  <tr>
    <td valign="top" width="50%">
      <table border="0" cellpadding="2" cellspacing="2" width="100%">
        <tr>
          <td>Your&nbsp;name:</td>
          $table_spacer
          <td colspan="2"><input type="text" name="tester_name" id="tester_name" value="$field_contents{'NAME'}" style="width: 13em;" /></td>
        </tr>
        <tr>
          <td>Organization:</td>
          $table_spacer
          <td colspan="2"><input type="text" name="organization" id="organization" value="$field_contents{'ORGANIZATION'}" style="width: 13em;" /></td>
        </tr>
        <tr>
          <td>E-mail:</td>
          $table_spacer
          <td><input type="text" name="tester_email" id="tester_email" value="$field_contents{'EMAIL'}" style="width: 13em;" /></td>
          <td width="100%" style="white-space: nowrap;"><label><input type="checkbox" name="send_email_report" id="send_email_report"$checkbox_state[$default_profile_general{SEND_EMAIL}] /> Send report via E-mail</label></td>
        </tr>
        <tr>
          <td colspan="4" style="border: dashed 1px gray; background: #ffc000; cursor: pointer;" onclick="javascript:show_hide_details();">&nbsp;<img src="images/arr_r.png" alt="" id="block_arrow_img" width="12" height="12" />&nbsp;&nbsp;Additional options</td>
        </tr>
        <tr>
          <td colspan="4" id="additional_data" style="border: dashed 1px gray; border-top: none; display: none; padding-right: 6px;">
            <table border="0" cellpadding="2" cellspacing="2" width="100%">
              <tr>
                <td>Verbose&nbsp;Level:</td>
                $table_spacer
                <td width="100%">
                  <select name="verbose_level" id="verbose_level" style="width: 13em;">
DATA
print "                    <option value=\"1\"".(($default_profile_general{'VERBOSE_LEVEL'} == 1) ? " selected=\"selected\"" : "").">1 (normal)</option>\n";
print "                    <option value=\"2\"".(($default_profile_general{'VERBOSE_LEVEL'} == 2) ? " selected=\"selected\"" : "").">2 (high)</option>\n";
print <<DATA;
                  </select>
                </td>
              </tr>
              <tr>
                <td colspan="3">Comments:</td>
              </tr>
              <tr>
                <td colspan="3">
                  <textarea name="comments" id="comments" rows="5" cols="68" style="width: 100%;">$field_contents{COMMENTS}</textarea>
                </td>
              </tr>
            </table>
          </td>
        </tr>
        <tr>
          <td colspan="4" valign="bottom" height="15"><hr /></td>
        </tr>
        <tr>
          <td colspan="4" valign="top" style="white-space: nowrap;"><label><input type="checkbox" name="use_internet" id="use_internet"$checkbox_state[$default_profile_general{USE_INTERNET}] onclick="javascript:update_elements_state();" ondblclick="javascript:setTimeout('update_elements_state()', 500);" />&nbsp;&nbsp;Use Internet for downloading the tests (download size: <b><span id="download_sz">0&nbsp;Kbytes</span></b>)</label></td>
        </tr>
        <tr>
          <td valign="middle" colspan="4">
            <table border="0" cellpadding="0" cellspacing="0" width="100%">
              <tr>
                <td>Architecture:</td>
                $table_spacer
                <td>
                  <select name="architecture" id="architecture" style="width: 6em;" onchange="javascript:show_modules();">
DATA

if (scalar(@supported_archs) == 0) {
	print "                    <option value=\"".$default_profile_general{'ARCHITECTURE'}."\" selected=\"selected\">".$default_profile_general{'ARCHITECTURE'}."</option>\n";
}
else {
	foreach (@supported_archs) {
		if ($_ eq $default_profile_general{'ARCHITECTURE'}) {
			print "                    <option value=\"$_\" selected=\"selected\">$_</option>\n";
		}
		else {
			print "                    <option value=\"$_\">$_</option>\n";
		}
	}
}

print <<DATA;
                  </select>
                </td>
                <td><img src="images/environment-spacer.gif" width="10" height="1" border="0" alt="" /></td>
                <td>Standard:</td>
                $table_spacer
                <td>
                  <select name="std_version" id="std_version" style="width: 9em;" onchange="javascript:show_modules();">
DATA

my $group = '';
foreach my $stdv (@all_std_versions) {
	my $curgroup = 'Unknown';
	if ($stdv =~ m/^(\S+) .*$/) {
		$curgroup = $1;
	}
	else {
		$curgroup = 'Unknown';
	}
	if ($group ne $curgroup) {
		if ($group ne '') {
			print "                    </optgroup>\n";
		}
		print "                    <optgroup label=\"$curgroup\">\n";
		$group = $curgroup;
	}
	if ($stdv eq $default_profile_general{'STD_VERSION'}) {
		print "                      <option value=\"$stdv\" selected=\"selected\">$stdv</option>\n";
	}
	else {
		print "                      <option value=\"$stdv\">$stdv</option>\n";
	}
}
if ($group ne '') {
	print "                    </optgroup>\n";
}

print <<DATA;
                  </select>
                </td>
                <td><img src="images/environment-spacer.gif" width="100" height="1" border="0" alt="" /></td>
DATA
my $exp_style = '';
my $exp_warning = '';

if ($manifest) {
	my $time_delta = int((time() - $manifest->{'MANIFEST'}->{'TIMESTAMP'}) / (24 * 60 * 60));
	if ($time_delta >= 7) {
		$exp_style = " color: red; font-weight: bold;";
		$exp_warning = "        <tr id=\"outdated_warn\">\n          <td colspan=\"4\" align=\"center\"><b><font color=\"red\">WARNING:</font> <font color=\"blue\">Your local cache is <font color=\"red\">$time_delta</font> days old!<br />Please, update it using the \"Refresh List\" button.</font></b></td>\n        </tr>\n";
	}
	elsif (!$manifest->{TREE} || !%{$manifest->{TREE}}) {
		$exp_style = " color: red; font-weight: bold;";
		$exp_warning = "        <tr id=\"outdated_warn\">\n          <td colspan=\"4\" align=\"center\"><br/><b><font color=\"blue\">Please, update the test list using the \"Refresh List\" button.</font></b></td>\n        </tr>\n";
	}
}
else {
	$exp_style = " color: red; font-weight: bold;";
	$exp_warning = "        <tr id=\"outdated_warn\">\n          <td colspan=\"4\" align=\"center\"><b><font color=\"red\">WARNING:</font> <font color=\"blue\">The local Manifest file is missing!<br />Please, press the \"Refresh List\" button to download it.</font></b></td>\n        </tr>\n";
}

print <<DATA;
                  <td align="right" width="100%"><input type="button" name="refresh_list" id="refresh_list_button" value="Refresh List" style="width: 8em;$exp_style" title="Update the list of available tests from the Internet" onclick="javascript:onRefresh();" /></td>
              </tr>
            </table>
          </td>
        </tr>
$exp_warning      </table>
DATA

print "      <div id=\"modlist_block\">\n";
print generate_modlist();
print "      </div>\n      <br />\n";

print <<DATA;
      <table cellpadding="0" cellspacing="0" width="100%" style="border: solid 4px white;"><tr><td>
        &nbsp;<b><span class="active" onclick="javascript:var e=document.getElementById('warning');if (e) {e.style.display=(e.style.display=='none'?'':'none');}" title="Click here for details"><font color="red">WARNING!</font> <font color="green">Do not use the tests on any machine holding important data!</font></span></b><br />
        <div id="warning" style="display: none;"><br />
        <div style="border: 1px dashed #2f6fab; background-color: #f9f9f9; width: 90%; padding: 3px; font-family: monospace;">
          WARNING! PLEASE DO NOT USE THIS SCRIPT ON ANY MACHINE THAT HOLDS IMPORTANT DATA! The tests sometimes put a heavy stress on the tested computer, and in rare unfortunate cases they can crash the system or damage the stored data. They also are a security risk as they pass along the root password.
        </div><br />
        </div>
      </td></tr></table>
      <table cellpadding="0" cellspacing="0" width="100%" style="border: solid 4px white;"><tr><td>
        <input type="button" name="run_tests" id="run_tests_button" value="Run Selected Tests" style="width: 11em;" onclick="javascript:return do_action('run_tests');" /><!--&nbsp;&nbsp; (Estimated&nbsp;execution&nbsp;time:&nbsp;<b><span id="estim_time">0&nbsp;min</span></b>)-->
      </td></tr></table>
    </td>
    <td style="border-right: dashed 1px gray;"><img src="images/environment-spacer.gif" width="15" height="1" border="0" alt="" /></td>
    <td><img src="images/environment-spacer.gif" width="15" height="1" border="0" alt="" /></td>
DATA

my $profiles_options = "";
my @profiles_list;
if (opendir(DIR, $SERVER_PARAM{"APP_DATA"}."/profiles")) {
	@profiles_list = sort grep !/^[~\.]/, readdir(DIR);
	foreach (@profiles_list) {
		$profiles_options .= "                <option value=\"$_\">$_</option>\n";
	}
	closedir(DIR);
}

my $disabled_list = "";
if ($profiles_options eq "") {
	$profiles_options = "                <option value=\"\">&lt;No profiles present&gt;</option>\n";
	$disabled_list = " disabled=\"disabled\"";
}

my $profiles_list_num = scalar(@profiles_list);
if ($profiles_list_num < 2) {
	$profiles_list_num = 2;
}
elsif ($profiles_list_num > 6) {
	$profiles_list_num = 6;
}

my $current_std = $profile->{'GENERAL'}->{'STD_VERSION'};
$current_std =~ s/ /:/g;
my $current_arch = $profile->{'GENERAL'}->{'ARCHITECTURE'};

print <<DATA;
    <td valign="top" width="50%">
      <h2>User Profile Management&nbsp;&nbsp;&nbsp;<b><span class="active" onclick="javascript:show_hide_tooltip(this, 'profile_management', event);"><img src="images/question.png" width="16" height="17" alt="[?]" title="What's this?" /></span></b></h2>
      <table border="0" cellpadding="2" cellspacing="2" width="100%">
        <tr>
          <td colspan="3">Select the profile or type its name:</td>
        </tr>
        <tr>
          <td colspan="3">
            <table border="0" cellpadding="0" cellspacing="0"><tr>
              <td style="vertical-align: middle;">
                <input type="text" name="edit_profile_name" id="edit_profile_name" value="" style="width: 18em;" onkeypress="javascript:return onKeyPressEdit(event);" onfocus="javascript:style.borderColor=input_border_color;" />
              </td>
              <td style="vertical-align: middle;">
                <img src="images/drop_down.png" alt="v" width="16" height="20" id="drop_down_button" style="cursor: pointer;" onclick="javascript:onClickArrow();" />
              </td>
            </tr></table>
            <div id="hidden_div" style="position: absolute; z-index: +1; display: none;">
              <select name="list_profile_name" id="list_profile_name" size="$profiles_list_num" style="font-family: monospace; font-size: 100%; width: 19.7em;"$disabled_list onclick="javascript:onClickList();" onkeypress="javascript:return onKeyPressList(event);">
$profiles_options              </select>
            </div>
          </td>
        </tr>
        <tr>
          <td><input type="button" name="save_profile_button" id="save_profile_button" value="Save" style="width: 5.8em;" onclick="javascript:onSave();" /></td>
          <td><input type="button" name="load_profile_button" id="load_profile_button" value="Load" style="width: 5.8em;" onclick="javascript:onLoad();" /></td>
          <td width="100%"><input type="button" name="delete_profile_button" id="delete_profile_button" value="Delete" style="width: 5.8em;" onclick="javascript:onDelete();" /></td>
        </tr>
      </table>
    </td>
  </tr>
</table>
</form>

DATA

# Function for writing Perl->HTML->JS templates.
# Input argument is a text variable containing HTML template code.
# The function converts it into a JS single-quotes string representing the
# appropriate HTML layout which can be passed into elem.innerHTML or so.
# <<<smth>>> patterns contain plain JS code snippets and are put into the resultant
# JS code unchanged (surrounded with '+' string operator).
# Also, compactification is performed (excessive spaces and newlines are removed).
# Starting and trailing quotes are not added.
#
# Example: html_compact("<b>Item No. '<<<x['1']>>>'</b>") eq "<b>Item No.\'' + x['1'] + '\'</b>";
# Known issue: additional "+ ''" is added to the end of the line.
sub html_compact($) {
	my ($html) = @_;
	$html =~ s/\n\s*//g;
	my @parts = split(/<<<|>>>/, $html);
	my $raw_js = 0;
	foreach my $part (@parts) {
		if ($raw_js) {
			$part .= ' + \'';
		}
		else {
			$part =~ s/\'/\\\'/g;
			$part .= '\' + ';
		}
		$raw_js = !$raw_js;
	}
	return join('', @parts).'\'';
}

my $tmp_table_code;

print <<DATA;
<script language="javascript" type="text/javascript">
// <![CDATA[
// Preload images for shadow, various icons, buttons, etc.
preload_images(new Array('../css/arr_down.png', 'arr_d.png', 'expand.png', 'collapse.png', 'drop_down_pressed.png', 'close.png', 'ajax_progress.gif', 'arr_down.png', 'shadow_bc.png', 'shadow_bl.png', 'shadow_br.png', 'shadow_mc.png', 'shadow_ml.png', 'shadow_mr.png', 'shadow_tc.png', 'shadow_tl.png', 'shadow_tr.png', 'status_noncert.png', 'status_certold.png', 'status_cert.png', 'status_beta.png', 'status_snapshot.png', 'status_unknown.png', 'status_local.png', 'files_changed.png', 'files_disabled.png', 'files_normal.png', 'inst_permit.png', 'inst_permit_gray.png', 'inst_force.png', 'inst_force_gray.png', 'inst_lock.png', 'inst_lock_gray.png', 'download_permit.png', 'download_permit_gray.png', 'download_force.png', 'download_force_gray.png', 'download_lock.png', 'download_lock_gray.png', 'configure_answers.png', 'configure_answers_dis.png', 'configure_answers_changed.png', 'button_plus.png', 'button_minus.png'));

// Global arrays and scalars dynamically redefined via AJAX:
var ts_files;           // Information which set of files forms every specific version of every test suite.
var file_ts;            // Reversed array for ts_files. file_ts[file] is a list of test suites,
                        // at least one version of which uses this file.
var file_size;          // Size of each file used by test suites.
var file_downloaded;    // 0: not downloaded; 1: downloaded; -1: downloaded wrong file
var file_installed;     // undefined: not installed, 'ver': installed, '': not a package
var ts_options;         // Global options of the test suites.
var ts_expect_info;     // Automatic replies for test suite questions during execution.
var ts_file_dl;         // Download type for each specific file:
                        // 0 or undefined - download, if necessary
                        // 1              - force download
                        // 2              - do not download
var ts_file_inst;       // Installation type for each specific file:
                        // 0 or undefined - install, if necessary
                        // 1              - force reinstall
                        // 2              - do not deinstall
var mod_cat_list;       // List of test suites in each (sub)category to check/uncheck.
var check_list;         // List of subcategories in each category to check/uncheck in addition to test suites.
var mod_time;           // Execution time for each test suite (currently unused).
var mod_list;           // Full list of test suite names.
var current_arch_std;   // String containing current architecture and standard version.
var arch_list;          // List of supported architectures.
var stdv_list;          // List of supported standard versions.
var profiles_list;      // List of user profiles.

// Function to build file_ts from ts_files.
function build_file_ts() {
	file_ts = new Array();
	for (var arch_std in ts_files) {
		for (var ts in ts_files[arch_std]) {
			for (var status in ts_files[arch_std][ts]) {
				for (var ver in ts_files[arch_std][ts][status]) {
					for (var i in ts_files[arch_std][ts][status][ver]) {
						var file = ts_files[arch_std][ts][status][ver][i];
						if (!file_ts[file])
							file_ts[file] = new Array();
						file_ts[file].push(arch_std + '-' + ts);
					}
				}
			}
		}
	}
}

// First initialization of variables used later in AJAX
function init_data() {
	current_arch_std = '$current_arch-$current_std';
DATA

print generate_js_init_data();
print "\n";
print "\tts_file_inst = new Array();\n";
print "\tts_file_dl = new Array();\n";
print "\n";
print "\tprofiles_list = new Array(".join(', ', map("'$_'", @profiles_list)).");\n";

print <<DATA;
}

////////////////////////////////////////////////////////////////////////////////
// Graphical Interface

// Find and cache most of the necessary HTML document elements.
var edit_profile_name = document.getElementById('edit_profile_name');
var list_profile_name = document.getElementById('list_profile_name');
var hidden_div = document.getElementById('hidden_div');
var drop_down_button = document.getElementById('drop_down_button');
var main_form = document.getElementById('configure_test');
var version_selector = document.getElementById('select_version');
var dialog_shadow = document.getElementById('shadow');
var dialog_shadow_center = document.getElementById('shadow_center');
var use_internet = document.getElementById('use_internet');
if (!list_profile_name || !edit_profile_name || !hidden_div || !drop_down_button || !main_form || !version_selector || !dialog_shadow || !dialog_shadow_center || !use_internet)
	alert('Internal error: Cannot find essential HTML elements! JavaScript will not work correctly.\\nPlease, contact the authors at linux\@ispras.ru.');

var input_border_color = ((navigator.appName == 'Konqueror') ? '#cccccc' : '');

var just_shown_dialog = false;          // Indicates that the dialog was just shown (to prevent its hiding from onBodyClick)
var dialog = null;                      // The dialog itself
var dialog_caller = null;               // Element which caused the dialog to appear
var dialog_caller_class = null;         // Normal class name of the dialog caller (to be restored when the dialog is closed)

// Determines absolute dynamic position of an element (1 pixel below its left bottom corner).
// Used for displaying the dialog box just below the element.
function getPosition(elem) {
	if (!elem)
		return {x: -1, y: -1};
	var x = 0;
	var y = elem.offsetHeight;
	while (elem) {
		x += elem.offsetLeft;
		y += elem.offsetTop;
		elem = elem.offsetParent;
	}
	return {x: x, y: y};
}

// Calculates total size (full and downloaded) for all files of the specific version of a test suite.
// Output:
//   .full    - total size of the files
//   .disk    - total size of downloaded files
//   .left    - how much is to be downloaded
//   .percent - percentage of downloaded size from the total size
function getFullSize(arch_std, id, status, ver) {
	var files = ts_files[arch_std][id][status][ver];
	var full_sz = 0;
	var disk_sz = 0;
	var left_sz = 0;
	var disk_pc = 100;
	if (files) {
		for (var i in files) {
			if (!file_size[files[i]])
				continue;
			full_sz += file_size[files[i]];
			if (file_downloaded[files[i]])
				disk_sz += file_size[files[i]];
		}
		left_sz = full_sz - disk_sz;
		if (full_sz) {
			disk_pc = Math.round(disk_sz * 100 / full_sz);
			full_sz = Math.round(full_sz / 1024);
			disk_sz = Math.round(disk_sz / 1024);
			left_sz = Math.round(left_sz / 1024);
		}
	}
	return {full: full_sz, disk: disk_sz, left: left_sz, percent: disk_pc};
}

// Hides the previous dialog (if it was shown) and prepare data for
// displaying the new one.
// CALLER - the HTML element which caused the dialog to appear and (possibly)
// will change its appearance because of that.
function initDialog(caller) {
	if (dialog_caller && dialog_caller_class)
		dialog_caller.className = dialog_caller_class;
	dialog_shadow.style.display = 'none';
	if (dialog)
		dialog.style.display = 'none';
	if (!caller || (caller == dialog_caller)) {
		dialog_caller = null;
		dialog_caller_class = null;
		return false;
	}
	else {
		dialog_caller = caller;
		dialog_caller_class = caller.className;
		return true;
	}
}

// Shows the dialog at the specified position and adjusts the shadow below it.
function showDialog(show_dlg, pos) {
	dialog = show_dlg;
	dialog.style.left = pos.x + 'px';
	dialog.style.top = pos.y + 'px';
	dialog.style.display = '';
	just_shown_dialog = true;
	dialog_shadow.style.left = pos.x + 'px';
	dialog_shadow.style.top = pos.y + 'px';
	dialog_shadow.style.display = '';
	adjust_shadow_size();
}

// Changes the shadow size to correspond to the dialog shown
function adjust_shadow_size() {
	if (dialog) {
		dialog_shadow_center.width = dialog.offsetWidth - 15;
		dialog_shadow_center.height = dialog.offsetHeight - 15;
	}
}

////////////////////////////////////////////////////////////////////////////////
//  Drop-down List for User Profiles

var VK_ESC  = 27;
var VK_CR   = 13;
var VK_LF   = 10;
var VK_DOWN = 40;
var VK_TAB  = 9;

var just_shown_list = false;

function show_list() {
	hidden_div.style.display = '';
	drop_down_button.setAttribute('src', 'images/drop_down_pressed.png');
	just_shown_list = true;
}

function hide_list(set_focus) {
	if (set_focus)
		edit_profile_name.focus();
	hidden_div.style.display = 'none';
	drop_down_button.setAttribute('src', 'images/drop_down.png');
}

function onBodyClick() {
	if (hidden_div.style.display != 'none') {
		if (!just_shown_list)
			hide_list(false);
		just_shown_list = false;
	}
	if (!just_shown_dialog)
		initDialog(null);
	just_shown_dialog = false;
}

function onKeyPressList(event) {
	if (hidden_div.style.display == 'none')
		return true;
	if (event.keyCode == VK_ESC) {
		hide_list(true);
		return false;
	}
	else if ((event.keyCode == VK_CR) || (event.keyCode == VK_LF)) {
		edit_profile_name.value = list_profile_name.value;
		hide_list(true);
		return false;
	}
	else if (event.keyCode == VK_TAB) {
		hide_list(false)
		return true;
	}
	return true;
}

function onClickList() {
	if (hidden_div.style.display == 'none')
		return true;
	edit_profile_name.value = list_profile_name.value;
	hide_list(true);
}

function onKeyPressEdit(event) {
	if (event.keyCode == VK_ESC) {
		if (hidden_div.style.display != 'none') {
			hide_list(true);
			return false;
		}
	}
	else if (event.keyCode == VK_DOWN) {
		if (hidden_div.style.display == 'none') {
			show_list();
		}
		list_profile_name.focus();
		return false;
	}
	else if (event.keyCode == VK_TAB) {
		if (hidden_div.style.display != 'none') {
			list_profile_name.focus();
			return false;
		}
	}
	else
		hide_list(true);
	return true;
}

function onClickArrow() {
	if (hidden_div.style.display == 'none') {
		show_list();
		list_profile_name.focus();
	}
	else {
		hide_list(true);
	}
}

////////////////////////////////////////////////////////////////////////////////
// Work with Test Suites

// Icons and text values
var version_statuses = new Array('cert', 'certold', 'noncert', 'beta', 'snapshot', 'local');
var version_status = new Array();
version_status['cert'] = '<font color="green"><b>Certifying</b></font>';
version_status['certold'] = '<font color="#004000"><b>Certifying (obsolete)</b></font>';
version_status['noncert'] = '<font color="#000080"><b>Stable, non-certifying</b></font>';
version_status['beta'] = '<font color="darkred">Beta</font>';
version_status['snapshot'] = '<font color="red">Snapshot</font>';
version_status['local'] = '<font color="#800080">Locally installed</font>';
var inst_dl_suffix = new Array();
inst_dl_suffix[0] = 'permit';
inst_dl_suffix[1] = 'force';
inst_dl_suffix[2] = 'lock';

// Splits the combined info 'status|version'.
function split_status_version(str) {
	var pos = str.indexOf('|');
	return {status: str.substr(0, pos), version: str.substr(pos + 1)};
}

function show_hide_inst_options(img_elem, details_id) {
	var elem = document.getElementById(details_id);
	var collapsed = (elem.style.display == 'none');
	if (collapsed) {
		elem.style.display = '';
		img_elem.setAttribute('src', 'images/button_minus.png');
		img_elem.setAttribute('alt', '[-]');
	}
	else {
		elem.style.display = 'none';
		img_elem.setAttribute('src', 'images/button_plus.png');
		img_elem.setAttribute('alt', '[+]');
	}
	adjust_shadow_size();
}

// Shows the dialog to select version of the test suite defined by ID.
function selectVersion(elem_parent, id) {
	if (!initDialog(elem_parent))
		return;
	elem_parent.className = 'dropdownh';
DATA

$tmp_table_code = html_compact(<<DATA);
<table class="main" style="width: 27em;">
  <tr>
    <th style="width: 12em;">Status</th>
    <th style="width: 5em;">Version</th>
    <th colspan="2">Downloaded</th>
  </tr>
DATA

print <<DATA;
	var res = '$tmp_table_code';
	var ver_elem = document.getElementById('ver-' + current_arch_std + '-' + id);
	var cur_ver = '';
	var cur_stat = '';
	if (ver_elem) {
		var sv = split_status_version(ver_elem.value);
		cur_ver = sv.version;
		cur_stat = sv.status;
	}
	for (var i=0; i<version_statuses.length; ++i) {
		var status = version_statuses[i];
		var ver;
		var vers_list = new Array();
		for (ver in ts_files[current_arch_std][id][status]) {
			vers_list.push(ver);
		}
		if (vers_list.length > 1) {
DATA

$tmp_table_code = html_compact(<<DATA);
  <tr>
    <td rowspan="<<<(vers_list.length + 1)>>>" style="vertical-align: top; background-color: #eeeeee;">
      <img src="images/status_<<<status>>>.png" alt="" width="16" height="16" />&nbsp;<<<version_status[status]>>>
    </td>
    <td colspan="3" style="display: none;">
    </td>
  </tr>
DATA

print <<DATA;
			res += '$tmp_table_code';
			for (ver in ts_files[current_arch_std][id][status]) {
				var sizes = getFullSize(current_arch_std, id, status, ver);
				var offline_enabled = (sizes.left == 0);
				if (!offline_enabled)
					sizes.left = '<b>' + sizes.left + '</b>';
				sizes.left += ' Kb left';
				if (!use_internet.checked && !offline_enabled)
					sizes.left = '<font color="red">' + sizes.left + '</font>';
				var disp_ver = ver;
				if (ver == '*') {
					var tmp = ts_local_ver[current_arch_std + '|' + id + '|' + status];
					disp_ver = tmp ? tmp : 'not installed';
				}
DATA

$tmp_table_code = html_compact(<<DATA);
  <tr class="selectable" onclick="javascript:acceptVersion('<<<id>>>', '<<<status>>>', '<<<ver>>>', true);">
    <td><<<(((ver == cur_ver) && (status == cur_stat)) ? ('<font color="blue"><b>' + disp_ver + '</b></font>') : disp_ver)>>></td>
    <td style="text-align: right;"><<<sizes.percent>>>%</td>
    <td style="text-align: right;"><<<sizes.left>>></td>
  </tr>
DATA

print <<DATA;
				res += '$tmp_table_code';
			}
		}
		else if (vers_list.length == 1) {
			ver = vers_list[0];
			var sizes = getFullSize(current_arch_std, id, status, ver);
			var offline_enabled = (sizes.left == 0);
			if (!offline_enabled)
				sizes.left = '<b>' + sizes.left + '</b>';
			sizes.left += ' Kb left';
			if (!use_internet.checked && !offline_enabled)
				sizes.left = '<font color="red">' + sizes.left + '</font>';
			var disp_ver = ver;
			if (ver == '*') {
				var tmp = ts_local_ver[current_arch_std + '|' + id + '|' + status];
				disp_ver = tmp ? tmp : 'not installed';
			}
DATA

$tmp_table_code = html_compact(<<DATA);
  <tr class="selectable" onclick="javascript:acceptVersion('<<<id>>>', '<<<status>>>', '<<<ver>>>', true);">
    <td><img src="images/status_<<<status>>>.png" alt="" width="16" height="16" />&nbsp;<<<version_status[status]>>></td>
    <td><<<(((ver == cur_ver) && (status == cur_stat)) ? ('<font color="blue"><b>' + disp_ver + '</b></font>') : disp_ver)>>></td>
    <td style="text-align: right;"><<<sizes.percent>>>%</td>
    <td style="text-align: right;"><<<sizes.left>>></td>
  </tr>
DATA

print <<DATA;
			res += '$tmp_table_code';
		}
	}
	res += '</table>';
	version_selector.innerHTML = res;
	var pos = getPosition(elem_parent);
	showDialog(version_selector, pos);
}

// Accepts the version selected
function acceptVersion(id, stat, ver, update) {
	var ver_elem = document.getElementById('ver-' + current_arch_std + '-' + id);
	var vis_elem = document.getElementById('visver-' + current_arch_std + '-' + id);
	var stat_elem = document.getElementById('stat-' + current_arch_std + '-' + id);
	if (ver_elem && vis_elem && stat_elem) {
		if (check_array_elem(ts_files, current_arch_std, id, stat, ver)) {
			ver_elem.value = stat + '|' + ver;
			var highlight = false;
			var disp_ver = ver;
			if (disp_ver == '*') {
				var tmp = ts_local_ver[current_arch_std + '|' + id + '|' + stat];
				if (tmp) {
					disp_ver = tmp;
				}
				else {
					disp_ver = 'not installed';
					highlight = true;
				}
			}
			if (!use_internet.checked) {
				var sizes = getFullSize(current_arch_std, id, stat, ver);
				if (sizes.left > 0) {
					highlight = true;
				}
			}
			if (highlight) {
				var elem = document.getElementById('mod-' + current_arch_std + '-' + id);
				if (elem.checked)
					disp_ver = '<font color="red"><b>' + disp_ver + '</b></font>';
				else
					disp_ver = '<font color="#C00000">' + disp_ver + '</font>';
			}
			vis_elem.innerHTML = disp_ver;
			stat_elem.setAttribute('src', 'images/status_' + stat + '.png');
			checkInstall(current_arch_std + '-' + id);
			checkCustomize(id);
		}
	}
	if (update) {
		initDialog(null);
		update_elements_state();
	}
}

// Shows the dialog for customizing installation type for all files at once.
function selectVersionAll(elem_parent) {
	if (!initDialog(elem_parent))
		return;
DATA

my $local_status = '';
if (in_array('local', @{$manifest->{ALL}{STATUSES}})) {
	$local_status = <<DATA;
      <br />
      <img src="images/status_local.png" alt="" width="16" height="16" />&nbsp;<span class="active" onclick="javascript:changeVersionAll('local');">Locally installed</span>
DATA
}

$tmp_table_code = html_compact(<<DATA);
<b>Select all latest versions that are:</b>
<table border="0" cellpadding="2" cellspacing="2" style="white-space: nowrap; min-width: 12em;">
  <tr>
    <td width="100%" style="border: none;">
      <img src="images/status_cert.png" alt="" width="16" height="16" />&nbsp;<span class="active" onclick="javascript:changeVersionAll('cert');">Certifying</span><br />
      <img src="images/status_noncert.png" alt="" width="16" height="16" />&nbsp;<span class="active" onclick="javascript:changeVersionAll('noncert');">Stable, but not certifying</span><br />
      <img src="images/status_beta.png" alt="" width="16" height="16" />&nbsp;<span class="active" onclick="javascript:changeVersionAll('beta');">Beta</span><br />
      <img src="images/status_snapshot.png" alt="" width="16" height="16" />&nbsp;<span class="active" onclick="javascript:changeVersionAll('snapshot');">Snapshot</span>
$local_status
    </td>
  </tr>
</table>
DATA

print <<DATA;
	version_selector.innerHTML = '$tmp_table_code';
	var pos = getPosition(elem_parent);
	showDialog(version_selector, pos);
}

// Switch installation type for all the files of all the test suites.
function changeVersionAll(status) {
	var max_ver;
	for (var ts in ts_files[current_arch_std]) {
		max_ver = '';
		for (var ver in ts_files[current_arch_std][ts][status]) {
			if (ver > max_ver)
				max_ver = ver;
		}
		if (max_ver) {
			var ver_elem = document.getElementById('ver-' + current_arch_std + '-' + ts);
			var vis_elem = document.getElementById('visver-' + current_arch_std + '-' + ts);
			var stat_elem = document.getElementById('stat-' + current_arch_std + '-' + ts);
			if (ver_elem && vis_elem && stat_elem) {
				ver_elem.value = status + '|' + max_ver;
				var disp_ver = max_ver;
				if (disp_ver == '*') {
					var tmp = ts_local_ver[current_arch_std + '|' + ts + '|' + status];
					disp_ver = tmp ? tmp : 'not installed';
				}
				vis_elem.innerHTML = disp_ver;
				stat_elem.setAttribute('src', 'images/status_' + status + '.png');
				checkInstall(current_arch_std + '-' + ts);
				checkCustomize(ts);
			}
		}
	}
	initDialog(null);
	update_elements_state();
}

// Shows the dialog for customizing installation type for each file the test suite consists of.
function selectInstall(elem_parent, id) {
	if (!initDialog(elem_parent))
		return;
	elem_parent.className = 'dropdownh';
	var res;
	var ver_elem = document.getElementById('ver-' + current_arch_std + '-' + id);
	var sv = split_status_version(ver_elem.value);
	var files = ts_files[current_arch_std][id][sv.status][sv.version];
	if (files && (files.length > 0)) {
DATA

$tmp_table_code = html_compact(<<DATA);
<span class="active" onclick="javascript:var e = document.getElementById('listed_files_options'); e.style.display = ((e.style.display == 'none') ? '' : 'none'); adjust_shadow_size();">Modify all listed files</span>
<div id="listed_files_options" style="display: none;">
<table border="0" cellpadding="2" cellspacing="2" width="100%" style="white-space: nowrap; border: dashed 1px gray;">
  <tr>
    <td width="50%" style="border: none; padding-left: 8px;">
      <img src="images/download_permit.png" alt="" width="16" height="16" />&nbsp;<span class="active" onclick="javascript:changeDownloadTS(0);">Download if needed</span><br />
      <img src="images/download_force.png" alt="" width="16" height="16" />&nbsp;<span class="active" onclick="javascript:changeDownloadTS(1);">Force download</span><br />
      <img src="images/download_lock.png" alt="" width="16" height="16" />&nbsp;<span class="active" onclick="javascript:changeDownloadTS(2);">Do not download</span>
    </td>
    <td width="50%" style="border: none; padding-right: 8px;">
      <img src="images/inst_permit.png" alt="" width="16" height="16" />&nbsp;<span class="active" onclick="javascript:changeInstallTS(0);">Install if needed</span><br />
      <img src="images/inst_force.png" alt="" width="16" height="16" />&nbsp;<span class="active" onclick="javascript:changeInstallTS(1);">Force reinstall</span><br />
      <img src="images/inst_lock.png" alt="" width="16" height="16" />&nbsp;<span class="active" onclick="javascript:changeInstallTS(2);">Do not deinstall</span>
    </td>
  </tr>
</table>
<br />
</div>
<form action="">
<table class="main">
  <tr>
    <th colspan="2">
      File Name
    </th>
    <th>
      File Size
    </th>
    <th>
      Downloaded
    </th>
    <th>
      Installed
    </th>
  </tr>
DATA

print <<DATA;
		res = '$tmp_table_code';
		for (var i=0; i<files.length; ++i) {
			// Is the file downloaded?
			var dl = '&mdash;';
			if (file_downloaded[files[i]] == 0) {
				if (use_internet.checked)
					dl = '<span style="color: darkred;">No</span>';
				else
					dl = '<span style="color: red;"><b>No</b></span>';
			}
			else if (file_downloaded[files[i]] == 1) {
				dl = '<span style="color: green;">Yes</span>';
			}
			else if (file_downloaded[files[i]] == -1) {
				dl = '<span style="color: darkred;">Old</span>';
			}
			// Is the file installed?
			var inst = '<span style="color: green;">Yes</span>';
			if (file_installed[files[i]] == '') {
				inst = '<span style="color: gray;">Not a package</span>';
			}
			else if (!file_installed[files[i]]) {
				// Package is not installed. This could be in one of the two following situations:
				if (file_size[files[i]]) {
					// The user selected an option an RPMed version, and the RPM package is not installed on the computer
					inst = '<span style="color: darkred;">No</span>';
				}
				else {
					// The user selected an option to run a locally installed package, but the package appeared to be not currently installed
					inst = '<span style="color: red;"><b>No</b></span>';
				}
			}

			// Calculate states of the file options
			var dl_opt = new Array(new Array(), new Array(), new Array());
			var inst_opt = new Array(new Array(), new Array(), new Array());
			for (var j = 0; j <= 2; ++j) {
				dl_opt[j]['disabled'] = '';
				dl_opt[j]['checked'] = '';
				inst_opt[j]['disabled'] = '';
				inst_opt[j]['checked'] = '';
			}
			var dl_sel = (ts_file_dl[files[i]] || 0);
			var inst_sel = (ts_file_inst[files[i]] || 0);

			if (!use_internet.checked) {
				dl_opt[0]['disabled'] = ' disabled="disabled"';
				dl_opt[1]['disabled'] = ' disabled="disabled"';
				if (file_downloaded[files[i]] == 0) {
					inst_opt[0]['disabled'] = ' disabled="disabled"';
					inst_opt[1]['disabled'] = ' disabled="disabled"';
				}
			}
			if (file_installed[files[i]] == '') {
				inst_opt[0]['disabled'] = ' disabled="disabled"';
				inst_opt[1]['disabled'] = ' disabled="disabled"';
				inst_opt[2]['disabled'] = ' disabled="disabled"';
			}
			else if (!file_installed[files[i]]) {
				inst_opt[2]['disabled'] = ' disabled="disabled"';
			}
			if ((file_downloaded[files[i]] == 0) && !file_installed[files[i]]) {
				dl_opt[2]['disabled'] = ' disabled="disabled"';
			}

			// If currently selected variant is disabled, set to a first non-disabled option (if any).
			// This is "mild" setting: to confirm it, user needs to click the option, else it is not saved.
			if (dl_opt[dl_sel]['disabled'] != '') {
				dl_sel = -1;
				for (var j = 0; j <= 2; ++j)
					if (dl_opt[j]['disabled'] == '') {
						dl_sel = j;
						break;
					}
			}
			if (inst_opt[inst_sel]['disabled'] != '') {
				inst_sel = -1;
				for (var j = 0; j <= 2; ++j)
					if (inst_opt[j]['disabled'] == '') {
						inst_sel = j;
						break;
					}
			}

			// Special case: file is installed, but not downloaded: the state depends on selection
			if (use_internet.checked) {
				if ((file_downloaded[files[i]] == 0) && file_installed[files[i]]) {
					if (dl_sel == 2) {
						inst_opt[0]['disabled'] = ' disabled="disabled"';
						inst_opt[1]['disabled'] = ' disabled="disabled"';
						inst_sel = 2;
					}
					else if (dl_sel == 1) {
						inst_opt[0]['disabled'] = ' disabled="disabled"';
						inst_opt[2]['disabled'] = ' disabled="disabled"';
						inst_sel = 1;
					}
					else {
						inst_opt[2]['disabled'] = ' disabled="disabled"';
						if (inst_sel == 2)
							inst_sel = 0;
					}
				}
				else {
					if (dl_sel == 1) {
						inst_opt[0]['disabled'] = ' disabled="disabled"';
						inst_opt[2]['disabled'] = ' disabled="disabled"';
						inst_sel = 1;
					}
				}
			}

			if (!file_size[files[i]]) {
				dl_opt[0]['disabled'] = ' disabled="disabled"';
				dl_opt[1]['disabled'] = ' disabled="disabled"';
				dl_opt[2]['disabled'] = ' disabled="disabled"';
				dl_sel = -1;
				inst_opt[0]['disabled'] = ' disabled="disabled"';
				inst_opt[1]['disabled'] = ' disabled="disabled"';
				inst_opt[2]['disabled'] = ' disabled="disabled"';
				inst_sel = -1;
			}
			// And finally put the checkmark into the radio-button calculated
			if (dl_sel != -1)
				dl_opt[dl_sel]['checked'] = ' checked="checked"';
			if (inst_sel != -1)
				inst_opt[inst_sel]['checked'] = ' checked="checked"';

			// Highlight problematic files
			var file_color = 'black';
			var error_tip = '';
			if ((dl_sel == -1) && (inst_sel == -1) && file_size[files[i]]) {
				// Use Internet is turned off, and the package is not downloaded
				file_color = 'red';
				error_tip = '<span style="color: blue;">Turn the <b>Use Internet</b> option on to get this file.</span>';
			}
			if (!file_size[files[i]] && !file_installed[files[i]]) {
				// A user selected an option to run locally installed package, but the package appeared to be not installed
				file_color = 'red';
				error_tip = '<span style="color: blue;">This package is expected to be installed.</span>';
			}

			// Calculate the icon for the file (whether the options are default or not)
			var file_icon = ((!ts_file_dl[files[i]] && !ts_file_inst[files[i]]) ? 'files_normal.png' : 'files_changed.png');

DATA

$tmp_table_code = html_compact(<<DATA);
  <tr>
    <td rowspan="2" valign="top" style="padding-top: 6px;">
      <img src="images/button_plus.png" alt="[+]" width="17" height="17" id="file_inst_img_<<<i>>>" style="cursor: pointer;" onclick="javascript:show_hide_inst_options(this, 'file_opt_<<<i>>>');" />
    </td>
    <td style="min-width: 8em; color: <<<file_color>>>;">
      <img src="images/<<<file_icon>>>" alt="" width="16" height="16" id="file_icon_<<<i>>>" />&nbsp;<<<files[i].substr(files[i].lastIndexOf('/') + 1)>>><input type="hidden" id="file_path_<<<i>>>" value="<<<files[i]>>>" />
    </td>
    <td align="right">
      <<<(file_size[files[i]] ? Math.round(file_size[files[i]] / 1024) + ' Kb' : '&mdash;')>>>
    </td>
    <td align="right">
      <<<dl>>>
    </td>
    <td align="right">
      <<<inst>>>
    </td>
  </tr>
  <tr>
    <td colspan="4" id="file_opt_<<<i>>>" style="display: none;">
      <<<error_tip>>>
      <table border="0" cellpadding="0" cellspacing="0" width="100%">
        <tr>
          <td width="50%" style="border: none; padding-left: 10px;">
            <label><input type="radio" name="file_opt_dl_<<<i>>>" id="file_opt_dl_<<<i>>>_permit" value="0"<<<dl_opt[0]['disabled']>>><<<dl_opt[0]['checked']>>> onclick="javascript:changeDownload('<<<files[i]>>>', <<<i>>>, 0);" />&nbsp;<img src="images/download_permit<<<((dl_opt[0]['disabled'] == '') ? '' : '_gray')>>>.png" alt="" width="16" height="16" />&nbsp;<span style="color: <<<((dl_opt[0]['disabled'] == '') ? 'black' : 'gray')>>>;">Download if needed</span><br /></label>
            <label><input type="radio" name="file_opt_dl_<<<i>>>" id="file_opt_dl_<<<i>>>_force" value="1"<<<dl_opt[1]['disabled']>>><<<dl_opt[1]['checked']>>> onclick="javascript:changeDownload('<<<files[i]>>>', <<<i>>>, 1);" />&nbsp;<img src="images/download_force<<<((dl_opt[1]['disabled'] == '') ? '' : '_gray')>>>.png" alt="" width="16" height="16" />&nbsp;<span style="color: <<<((dl_opt[1]['disabled'] == '') ? 'black' : 'gray')>>>;">Force download</span><br /></label>
            <label><input type="radio" name="file_opt_dl_<<<i>>>" id="file_opt_dl_<<<i>>>_lock" value="2"<<<dl_opt[2]['disabled']>>><<<dl_opt[2]['checked']>>> onclick="javascript:changeDownload('<<<files[i]>>>', <<<i>>>, 2);" />&nbsp;<img src="images/download_lock<<<((dl_opt[2]['disabled'] == '') ? '' : '_gray')>>>.png" alt="" width="16" height="16" />&nbsp;<span style="color: <<<((dl_opt[2]['disabled'] == '') ? 'black' : 'gray')>>>;">Do not download</span></label>
          </td>
          <td width="50%" style="border: none; padding-right: 10px;">
            <label><input type="radio" name="file_opt_inst_<<<i>>>" id="file_opt_inst_<<<i>>>_permit" value="0"<<<inst_opt[0]['disabled']>>><<<inst_opt[0]['checked']>>> onclick="javascript:changeInstall('<<<files[i]>>>', <<<i>>>, 0);" />&nbsp;<img src="images/inst_permit<<<((inst_opt[0]['disabled'] == '') ? '' : '_gray')>>>.png" alt="" width="16" height="16" id="file_opt_inst_<<<i>>>_img_permit" />&nbsp;<span style="color: <<<((inst_opt[0]['disabled'] == '') ? 'black' : 'gray')>>>;" id="file_opt_inst_<<<i>>>_txt_permit">Install if needed</span><br /></label>
            <label><input type="radio" name="file_opt_inst_<<<i>>>" id="file_opt_inst_<<<i>>>_force" value="1"<<<inst_opt[1]['disabled']>>><<<inst_opt[1]['checked']>>> onclick="javascript:changeInstall('<<<files[i]>>>', <<<i>>>, 1);" />&nbsp;<img src="images/inst_force<<<((inst_opt[1]['disabled'] == '') ? '' : '_gray')>>>.png" alt="" width="16" height="16" id="file_opt_inst_<<<i>>>_img_force" />&nbsp;<span style="color: <<<((inst_opt[1]['disabled'] == '') ? 'black' : 'gray')>>>;" id="file_opt_inst_<<<i>>>_txt_force">Force reinstall</span><br /></label>
            <label><input type="radio" name="file_opt_inst_<<<i>>>" id="file_opt_inst_<<<i>>>_lock" value="2"<<<inst_opt[2]['disabled']>>><<<inst_opt[2]['checked']>>> onclick="javascript:changeInstall('<<<files[i]>>>', <<<i>>>, 2);" />&nbsp;<img src="images/inst_lock<<<((inst_opt[2]['disabled'] == '') ? '' : '_gray')>>>.png" alt="" width="16" height="16" id="file_opt_inst_<<<i>>>_img_lock" />&nbsp;<span style="color: <<<((inst_opt[2]['disabled'] == '') ? 'black' : 'gray')>>>;" id="file_opt_inst_<<<i>>>_txt_lock">Do not deinstall</span></label>
          </td>
        </tr>
      </table>
    </td>
  </tr>
DATA

print <<DATA;
			res += '$tmp_table_code';
		}
		res += '</table></form>';
	}
	else {
		res = '<i>This test suite version has no files<br />to be customized.</i>';
	}
	version_selector.innerHTML = res;
	var pos = getPosition(elem_parent);
	showDialog(version_selector, pos);
}

// The function adjusts state of the options according to the selected values, if necessary
function controlInstallOptions(file, idx, dl) {
	// The function sets the disabled state, changes the icon and text color in one step
	function set_state(elem_idx, enable) {
		elem[elem_idx].disabled = !enable;
		img[elem_idx].setAttribute('src', 'images/inst_' + inst_dl_suffix[elem_idx] + (enable ? '' : '_gray') + '.png');
		txt[elem_idx].style.color = (enable ? 'black' : 'gray');
	}

	var inst = 0;
	var elem = new Array(3);
	var img = new Array(3);
	var txt = new Array(3);
	for (var i = 0; i <= 2; ++i) {
		elem[i] = document.getElementById('file_opt_inst_' + idx + '_' + inst_dl_suffix[i]);
		img[i] = document.getElementById('file_opt_inst_' + idx + '_img_' + inst_dl_suffix[i]);
		txt[i] = document.getElementById('file_opt_inst_' + idx + '_txt_' + inst_dl_suffix[i]);
		if (elem[i].checked)
			inst = i;
	}

	// All currently defined events are triggered only when Internet is available and
	// only with installable packages.
	if (use_internet.checked && (file_installed[file] != '')) {
		// File is installed, but not downloaded:
		// if user selects not to download the file, then forbid deinstalling;
		// if user selects downloading, then force (re)installing.
		if ((file_downloaded[file] == 0) && file_installed[file]) {
			if (dl == 0) {
				set_state(0, true);
				set_state(1, true);
				set_state(2, false);
				if (elem[2].checked) {
					elem[2].checked = false;
					elem[0].checked = true;
					inst = 0;
				}
			}
			else if (dl == 1) {
				set_state(0, false);
				set_state(1, true);
				set_state(2, false);
				elem[0].checked = false;
				elem[1].checked = true;
				elem[2].checked = false;
				inst = 1;
			}
			else if (dl == 2) {
				set_state(0, false);
				set_state(1, false);
				set_state(2, true);
				elem[0].checked = false;
				elem[1].checked = false;
				elem[2].checked = true;
				inst = 2;
			}
		}
		else {
			var allow_no_deinstall = file_installed[file];
			// If user selects forced downloading, he is only allowed to force reinstalling
			if (dl == 1) {
				set_state(0, false);
				set_state(1, true);
				set_state(2, false);
				elem[0].checked = false;
				elem[1].checked = true;
				elem[2].checked = false;
				inst = 1;
			}
			else {
				set_state(0, true);
				set_state(2, allow_no_deinstall);
			}
		}
		ts_file_inst[file] = inst;
	}
}

// Switch download type for the file selected.
function changeDownload(file, idx, dl) {
	ts_file_dl[file] = dl;
	// Update the option states
	controlInstallOptions(file, idx, dl);
	document.getElementById('file_icon_' + idx).setAttribute('src', 'images/' + ((!ts_file_dl[file] && !ts_file_inst[file]) ? 'files_normal.png' : 'files_changed.png'));
	for (var i in file_ts[file]) {
		checkInstall(file_ts[file][i]);
	}
	update_elements_state();
}

// Switch installation type for the file selected.
function changeInstall(file, idx, inst) {
	ts_file_inst[file] = inst;
	document.getElementById('file_icon_' + idx).setAttribute('src', 'images/' + ((!ts_file_dl[file] && !ts_file_inst[file]) ? 'files_normal.png' : 'files_changed.png'));
	for (var i in file_ts[file]) {
		checkInstall(file_ts[file][i]);
	}
}

// Switch download type for all the files of the test suite.
function changeDownloadTS(dl) {
	var i = 0;
	var ids = new Array();
	var file_radios = new Array();
	while (true) {
		var file_path = document.getElementById('file_path_' + i);
		if (file_path) {
			for (var j = 0; j <= 2; ++j)
				file_radios[j] = document.getElementById('file_opt_dl_' + i + '_' + inst_dl_suffix[j]);
			if (!file_radios[dl].disabled) {
				ts_file_dl[file_path.value] = dl;
				// Update the option states
				controlInstallOptions(file_path.value, i, dl);
				for (var j = 0; j <= 2; ++j)
					file_radios[j].checked = (j == dl);
				for (var j in file_ts[file_path.value]) {
					ids[file_ts[file_path.value][j]] = true;
				}
				document.getElementById('file_icon_' + i).setAttribute('src', 'images/' + ((!ts_file_dl[file_path.value] && !ts_file_inst[file_path.value]) ? 'files_normal.png' : 'files_changed.png'));
			}
			++i;
		}
		else
			break;
	}
	for (var id in ids)
		checkInstall(id);
	update_elements_state();
}

// Switch installation type for all the files of the test suite.
function changeInstallTS(inst) {
	var i = 0;
	var ids = new Array();
	var file_radios = new Array();
	while (true) {
		var file_path = document.getElementById('file_path_' + i);
		if (file_path) {
			for (var j = 0; j <= 2; ++j)
				file_radios[j] = document.getElementById('file_opt_inst_' + i + '_' + inst_dl_suffix[j]);
			if (!file_radios[inst].disabled) {
				ts_file_inst[file_path.value] = inst;
				for (var j = 0; j <= 2; ++j)
					file_radios[j].checked = (j == inst);
				for (var j in file_ts[file_path.value]) {
					ids[file_ts[file_path.value][j]] = true;
				}
				document.getElementById('file_icon_' + i).setAttribute('src', 'images/' + ((!ts_file_dl[file_path.value] && !ts_file_inst[file_path.value]) ? 'files_normal.png' : 'files_changed.png'));
			}
			++i;
		}
		else
			break;
	}
	for (var id in ids)
		checkInstall(id);
}

// Shows the dialog for customizing installation and download type for all files at once.
function fileOptionsAll(elem_parent) {
	if (!initDialog(elem_parent))
		return;
DATA

$tmp_table_code = html_compact(<<DATA);
<b>Apply to all files:</b>
<table border="0" cellpadding="2" cellspacing="2" style="white-space: nowrap; min-width: 24em;">
  <tr>
    <td width="50%" style="border: none;">
      <img src="images/download_permit.png" alt="" width="16" height="16" />&nbsp;<span class="active" onclick="javascript:changeDownloadAll(0);">Download if needed</span><br />
      <img src="images/download_force.png" alt="" width="16" height="16" />&nbsp;<span class="active" onclick="javascript:changeDownloadAll(1);">Force download</span><br />
      <img src="images/download_lock.png" alt="" width="16" height="16" />&nbsp;<span class="active" onclick="javascript:changeDownloadAll(2);">Do not download</span>
    </td>
    <td width="50%" style="border: none;">
      <img src="images/inst_permit.png" alt="" width="16" height="16" />&nbsp;<span class="active" onclick="javascript:changeInstallAll(0);">Install if needed</span><br />
      <img src="images/inst_force.png" alt="" width="16" height="16" />&nbsp;<span class="active" onclick="javascript:changeInstallAll(1);">Force reinstall</span><br />
      <img src="images/inst_lock.png" alt="" width="16" height="16" />&nbsp;<span class="active" onclick="javascript:changeInstallAll(2);">Do not deinstall</span>
    </td>
  </tr>
</table>
DATA

print <<DATA;
	version_selector.innerHTML = '$tmp_table_code';
	var pos = getPosition(elem_parent);
	showDialog(version_selector, pos);
}

// Switch installation type for all the files of all the test suites.
function changeDownloadAll(dl) {
	var ids = new Array();
	for (var ts in ts_files[current_arch_std]) {
		for (var status in ts_files[current_arch_std][ts]) {
			for (var ver in ts_files[current_arch_std][ts][status]) {
				for (var i in ts_files[current_arch_std][ts][status][ver]) {
					var file = ts_files[current_arch_std][ts][status][ver][i];
					ts_file_dl[file] = dl;
				}
			}
		}
		ids[current_arch_std + '-' + ts] = true;
	}
	for (var id in ids)
		checkInstall(id);
	update_elements_state();
	initDialog(null);
}

// Switch installation type for all the files of all the test suites.
function changeInstallAll(inst) {
	var ids = new Array();
	for (var ts in ts_files[current_arch_std]) {
		for (var status in ts_files[current_arch_std][ts]) {
			for (var ver in ts_files[current_arch_std][ts][status]) {
				for (var i in ts_files[current_arch_std][ts][status][ver]) {
					var file = ts_files[current_arch_std][ts][status][ver][i];
					ts_file_inst[file] = inst;
				}
			}
		}
		ids[current_arch_std + '-' + ts] = true;
	}
	for (var id in ids)
		checkInstall(id);
	initDialog(null);
}

// Updates the status of the test suite (default or customized), depending on the
// state of all its files.
function checkInstall(arch_std_id) {
	var vis_elem = document.getElementById('visinst-' + arch_std_id);
	if (vis_elem) {
		var changed = false;
		var ver_elem = document.getElementById('ver-' + arch_std_id);
		var pos = arch_std_id.indexOf('-');
		pos = arch_std_id.indexOf('-', pos + 1);
		if (arch_std_id.substr(0, 6) == 'x86-64')
			pos = arch_std_id.indexOf('-', pos + 1);
		var id = arch_std_id.substr(pos + 1);
		var arch_std = arch_std_id.substr(0, pos);
		var sv = split_status_version(ver_elem.value);
		var files = ts_files[arch_std][id][sv.status][sv.version];
		if (files && (files.length > 0)) {
			for (var i=0; i<files.length; ++i) {
				if (ts_file_inst[files[i]] || ts_file_dl[files[i]]) {
					changed = true;
					break;
				}
			}
			vis_elem.setAttribute('src', (changed ? 'images/files_changed.png' : 'images/files_normal.png'));
		}
		else
			vis_elem.setAttribute('src', 'images/files_disabled.png');
	}
}

// Shows the dialog for customizing automatic replies for test suite questions.
function customizeTS(elem_parent, id) {
	if (!initDialog(elem_parent))
		return;
	elem_parent.className = 'dropdownh';
	var ver_elem = document.getElementById('ver-' + current_arch_std + '-' + id);
	var valid = false;
	if (ver_elem) {
		var sv = split_status_version(ver_elem.value);
		valid = true;
	}
	var list = ts_expect_info[current_arch_std][id][sv.status][sv.version];
	var num = 0;
	var res = '';
	if (list)
		for (var qid in list) {
			if (num == 0) {
				var hr_style = '';
				if (navigator.appName == 'Microsoft Internet Explorer') {
					hr_style = 'style="width: 38em;" ';
				}
DATA

$tmp_table_code = html_compact(<<DATA);
<b>Automatic replies for test suite questions:</b>
<hr <<<hr_style>>>/>
<div id="customize_div" style="font-size: 80%; height: auto; max-height: 30em; overflow: auto;">
  <table class="main" id="customize_table" style="margin: 2px;">
DATA

print <<DATA;
				res = '$tmp_table_code';
			}
			++num;
DATA

$tmp_table_code = html_compact(<<DATA);
    <tr>
      <td style="width:30em; white-space: normal;"><<<list[qid].question>>></td>
      <td><input type="text" name="field_<<<qid>>>" id="field_<<<qid>>>" value="<<<list[qid].answer>>>" style="width: 13em;" /></td>
      <td><img src="images/inst_permit.png" alt="" width="16" height="16" title="Reset to default" style="cursor: pointer;" onclick="javascript:document.getElementById('field_<<<qid>>>').value='<<<list[qid].def_answer>>>';" /></td>
    </tr>
DATA

print <<DATA;
			res += '$tmp_table_code';
		}
	if (num > 0) {
DATA

$tmp_table_code = html_compact(<<DATA);
  </table>
</div>
<hr <<<hr_style>>>/><br />
<div style="text-align: right;">
  <input type="button" value="Accept" style="width: 7em;" onclick="javascript:var list=ts_expect_info['<<<current_arch_std>>>']['<<<id>>>']['<<<sv.status>>>']['<<<sv.version>>>'];for (var qid in list) list[qid].answer=document.getElementById('field_' + qid).value; checkCustomize('<<<id>>>'); initDialog(null);" />&nbsp;
  <input type="button" value="Cancel" style="width: 7em;" onclick="javascript:initDialog(null);" />&nbsp;
  <input type="button" value="Reset" style="width: 7em;" onclick="javascript:var list=ts_expect_info['<<<current_arch_std>>>']['<<<id>>>']['<<<sv.status>>>']['<<<sv.version>>>'];for (var qid in list) document.getElementById('field_' + qid).value=list[qid].def_answer;" />
</div>
DATA

print <<DATA;
		res += '$tmp_table_code';
	}
	else {
		res = '<i>This test suite version has no customizable options.</i>';
	}
	version_selector.innerHTML = res;
	var pos = getPosition(elem_parent);
	showDialog(version_selector, pos);
	// Fix IE problem: scrollbar (if present) obscures part of the table (have to scroll horizontally as well).
	if (navigator.appName == 'Microsoft Internet Explorer') {
		var table = document.getElementById('customize_table');
		var div = document.getElementById('customize_div');
		if (table && div) {
			if (table.clientHeight > div.clientHeight)
				table.style.marginRight = '20px';
		}
	}
}

// Updates the status of the test suite (default or customized), depending on the
// state of all its files.
function checkCustomize(id) {
	var vis_elem = document.getElementById('viscust-' + current_arch_std + '-' + id);
	if (vis_elem) {
		var changed = false;
		var ver_elem = document.getElementById('ver-' + current_arch_std + '-' + id);
		var sv = split_status_version(ver_elem.value);
		var list = ts_expect_info[current_arch_std][id][sv.status][sv.version];
		var num = 0;
		if (list) {
			for (var qid in list) {
				++num;
				if (list[qid].answer != list[qid].def_answer) {
					changed = true;
					break;
				}
			}
		}
		if (num > 0)
			vis_elem.setAttribute('src', (changed ? 'images/configure_answers_changed.png' : 'images/configure_answers.png'));
		else
			vis_elem.setAttribute('src', 'images/configure_answers_dis.png');
	}
}

// Shows the dialog for resetting automatic replies for all test suites to default values.
function customizeAll(elem_parent) {
	if (!initDialog(elem_parent))
		return;
DATA

$tmp_table_code = html_compact(<<DATA);
<b>Apply to all test suites:</b>
<table border="0" cellpadding="2" cellspacing="2" style="white-space: nowrap; min-width: 12em;">
  <tr>
    <td width="100%" style="border: none;">
      <img src="images/inst_permit.png" alt="" width="16" height="16" /> <span class="active" onclick="javascript:resetCustomizeAll();">Reset to default</span>
    </td>
  </tr>
</table>
DATA

print <<DATA;
	version_selector.innerHTML = '$tmp_table_code';
	var pos = getPosition(elem_parent);
	showDialog(version_selector, pos);
}

// Resets automatic replies for all test suites.
function resetCustomizeAll() {
	for (var ts in ts_expect_info[current_arch_std]) {
		for (var status in ts_expect_info[current_arch_std][ts]) {
			for (var ver in ts_expect_info[current_arch_std][ts][status]) {
				var list = ts_expect_info[current_arch_std][ts][status][ver];
				if (list)
					for (var qid in list)
						list[qid].answer = list[qid].def_answer;
			}
		}
		checkCustomize(ts);
	}
	initDialog(null);
}

// Updates view of test suite versions and total size to be downloaded.
function update_elements_state() {
	var elem;
	var i, j;
	var files_to_dl = new Array();
	var download_sz = 0;
//	var total_time = 0;
	var total_num = 0;

	// Enable/disable 'Refresh List' button (as it needs Internet connection)
	elem = document.getElementById('refresh_list_button');
	if (elem)
		elem.disabled = !use_internet.checked;

	// for each module...
	for (i=0; i<mod_list.length; ++i) {
		var id = current_arch_std + '-' + mod_list[i];
		// Find the module’s checkbox.
		elem = document.getElementById('mod-' + id);
		if (!elem)
			continue;

		// Enable/disable the options depending on whether module is checked or not.
		for (var j in ts_options[current_arch_std][mod_list[i]]) {
			var elemopt = document.getElementsByName('modopt-' + id + '-' + ts_options[current_arch_std][mod_list[i]][j]);
			for (j=0; j<elemopt.length; ++j) {
				elemopt[j].disabled = !elem.checked;
			}
		}

		// Calculate download size of test suites.
		var highlight = false;
		var ver_elem = document.getElementById('ver-' + id);
		var sv = split_status_version(ver_elem.value);
		var files = ts_files[current_arch_std][mod_list[i]][sv.status][sv.version];
		if (files) {
			for (var j in files) {
				if (file_downloaded[files[j]] == 0) {
					if (!use_internet.checked) {
						highlight = true;
					}
				}
				if (!file_size[files[j]] && !file_installed[files[j]]) {
					highlight = true;
				}
				if (elem.checked && !files_to_dl[files[j]]) {
					if ((ts_file_dl[files[j]] == 1) || (!ts_file_dl[files[j]] && (file_downloaded[files[j]] != 1))) {
						if (file_size[files[j]]) {
							files_to_dl[files[j]] = 1;
							download_sz += file_size[files[j]];
						}
					}
				}
			}
		}
		if (elem.checked) {
			++total_num;
		}
		var vis_elem = document.getElementById('visver-' + id);
		var disp_ver = sv.version;
		if (disp_ver == '*') {
			var tmp = ts_local_ver[current_arch_std + '|' + mod_list[i] + '|' + sv.status];
			disp_ver = tmp ? tmp : 'not installed';
		}
		if (highlight) {
			if (elem.checked)
				vis_elem.innerHTML = '<font color="red"><b>' + disp_ver + '</b></font>';
			else
				vis_elem.innerHTML = '<font color="#C00000">' + disp_ver + '</font>';
		}
		else {
			vis_elem.innerHTML = disp_ver;
		}
	}
	// Display the size of files to download together with the total time of running the tests.
	document.getElementById('download_sz').innerHTML = Math.round(download_sz / 1024) + '&nbsp;Kbytes';
	//document.getElementById('estim_time').innerHTML =  Math.round(total_time / 60) + '&nbsp;min';
	elem = document.getElementById('run_tests_button');
	if (elem)
		elem.disabled = (total_num == 0);
}

function update_all() {
	for (var i in mod_list) {
		checkCustomize(mod_list[i]);
		checkInstall(current_arch_std + '-' + mod_list[i]);
	}
	update_elements_state();
}

function un_check_all(category) {
	var elem = document.getElementById('check-' + category + '-' + current_arch_std);
	if (elem) {
		var checked = elem.checked;
		if (mod_cat_list[current_arch_std][category])
			for (var i=0; i<mod_cat_list[current_arch_std][category].length; ++i) {
				elem = document.getElementById('mod-' + current_arch_std + '-' + mod_cat_list[current_arch_std][category][i]);
				if (elem && !elem.disabled)
					elem.checked = checked;
			}
		if (check_list[current_arch_std][category])
			for (var i=0; i<check_list[current_arch_std][category].length; ++i) {
				elem = document.getElementById('check-' + check_list[current_arch_std][category][i] + '-' + current_arch_std);
				if (elem && !elem.disabled)
					elem.checked = checked;
			}
		if ((category == 'certification_tests') && checked) {
			elem = document.getElementById('modopt-' + current_arch_std + '-libchk-profile-core-c---desktop');
			if (elem)
				elem.checked = true;
			elem = document.getElementById('modopt-' + current_arch_std + '-desktop-with_qt4-desktop');
			if (elem)
				elem.checked = true;
			elem = document.getElementById('modopt-' + current_arch_std + '-cmdchk-profile-core-c---desktop');
			if (elem)
				elem.checked = true;
		}
		update_elements_state();
	}
}

function show_modules() {
	var i, j;
	var elem_a = document.getElementById('architecture');
	var elem_l = document.getElementById('std_version');
	if (!elem_a || !elem_l)
		return;
	current_arch_std = elem_a.value + '-' + elem_l.value.replace(/ /g, ':');
	for (i=0; i<stdv_list.length; i++) {
		for (j=0; j<arch_list.length; j++) {
			var tmp = arch_list[j] + '-' + stdv_list[i];
			var elem = document.getElementById('arch-' + tmp);
			if (elem) {
				if (tmp == current_arch_std) {
					elem.style.display = '';
				}
				else {
					elem.style.display = 'none';
				}
			}
		}
	}
	update_all();
}

////////////////////////////////////////////////////////////////////////////////
// Tooltips

var tooltip_texts = new Array();
tooltip_texts['certification_tests'] = 'Test suites that are used for certification of distributions<br />for conformance to the Standard Specifications.';
tooltip_texts['certification_tests-command_tests'] = 'Test suites that verify the commands<br />required by Standard Specification.';
tooltip_texts['certification_tests-static_interface_tests'] = 'Test suites that verify interfaces in the runtime libraries.';
tooltip_texts['certification_tests-runtime_interface_tests'] = 'Test suites for testing runtime environments<br />for conformance to the Standard Specifications.';
tooltip_texts['certification_tests-automated_application_battery_tests'] = 'Tests that verify the correct operation of binary applications<br />from the LSB/Moblin application battery.';
tooltip_texts['certification_tests-manual_tests'] = 'All the tests that need user interaction.';
tooltip_texts['unofficial_tests'] = 'Tests that are not used for certification,<br />but can be used for futher standard compliance testing.';
tooltip_texts['unofficial_tests-runtime_interface_tests'] = 'Test suites for testing runtime environments<br />for conformance to the Standard Specifications.';
tooltip_texts['profile_management'] = 'This section allows you to save all the settings<br />from the left pane into user profiles and load them afterwards.';

function show_hide_tooltip(elem_parent, id, event) {
	if (!initDialog(elem_parent))
		return;
	var tt_window = document.getElementById('tooltip');
	if (tt_window) {
		var tt_text = document.getElementById('tooltip_text');
		if (tt_text) {
			if (tooltip_texts[id])
				tt_text.innerHTML = tooltip_texts[id] + '<br /><div align="right"><a href="tests_help.pl#' + id + '" target="_blank" onclick="javascript:initDialog(null);"><i>Details...</i></a></div>';
			else
				tt_text.innerHTML = '<font color="red"><b>Sorry, no information for this item was found!</b></font><br />Please, write to <a href="mailto:linux\@ispras.ru">linux\@ispras.ru</a>.';
			var scrollTop = document.body.scrollTop;
			if (scrollTop == 0)
			{
				if (window.pageYOffset)
					scrollTop = window.pageYOffset;
				else
					scrollTop = (document.body.parentElement) ? document.body.parentElement.scrollTop : 0;
			}
			var scrollLeft = document.body.scrollLeft;
			if (scrollLeft == 0)
			{
				if (window.pageXOffset)
					scrollLeft = window.pageXOffset;
				else
					scrollLeft = (document.body.parentElement) ? document.body.parentElement.scrollLeft : 0;
			}
			var div_top = event.clientY + scrollTop + 10;
			var div_left = event.clientX + scrollLeft + 10;
			tt_window.style.display = '';
			var div_width = (tt_window.style.pixelWidth || tt_window.offsetWidth);
			var wnd_width = (document.body.clientWidth || window.innerWidth);
			if (div_left + div_width > wnd_width + scrollLeft)
				div_left = wnd_width + scrollLeft - div_width;
			if (div_left < scrollLeft)
				div_left = scrollLeft;
			showDialog(tt_window, {x: div_left, y: div_top});
		}
	}
}

////////////////////////////////////////////////////////////////////////////////
// User Profiles

function set_options_by_profile() {
DATA

print generate_profile_init($profile, 'conf');

print <<DATA;
}

var to_check_ts;
var to_check_ts_ver;
var to_check_ts_opt;

function save_profile_options_modules() {
	var elem, elem_ver, i, j;
	to_check_ts = new Array();
	to_check_ts_ver = new Array();
	to_check_ts_opt = new Array();
	for (i in mod_list) {
		var mod = mod_list[i];
		elem = document.getElementById('mod-' + current_arch_std + '-' + mod);
		elem_ver = document.getElementById('ver-' + current_arch_std + '-' + mod);
		if (elem && elem_ver) {
			to_check_ts[mod] = elem.checked;
			to_check_ts_ver[mod] = elem_ver.value;
			for (var j in ts_options[current_arch_std][mod]) {
				var opt_name = ts_options[current_arch_std][mod][j];
				var elemopt = document.getElementsByName('modopt-' + current_arch_std + '-' + mod + '-' + opt_name);
				if (elemopt.length > 0) {
					if (!to_check_ts_opt[mod])
						to_check_ts_opt[mod] = new Array();
					for (k in elemopt) {
						if (elemopt[k].checked)
							to_check_ts_opt[mod][opt_name] = elemopt[k].value;
					}
				}
			}
		}
	}
}

function apply_profile_options_modules() {
	var elem, elem_ver, elem_vis, elem_stat;
	var cat_list = new Array();
	// Make a copy of the array mod_cat_list (for current arch-std only).
	for (var cat in mod_cat_list[current_arch_std]) {
		cat_list[cat] = mod_cat_list[current_arch_std][cat].slice(0);
	}
	for (var mod in to_check_ts) {
		elem = document.getElementById('mod-' + current_arch_std + '-' + mod);
		elem_ver = document.getElementById('ver-' + current_arch_std + '-' + mod);
		elem_vis = document.getElementById('visver-' + current_arch_std + '-' + mod);
		elem_stat = document.getElementById('stat-' + current_arch_std + '-' + mod);
		if (elem && elem_ver && elem_vis && elem_stat) {
			var sv = split_status_version(to_check_ts_ver[mod]);
			sv.version = '';
			if (ts_files[current_arch_std][mod][sv.status]) {
				for (var ver in ts_files[current_arch_std][mod][sv.status]) {
					sv.version = ver;
					break;
				}
			}
			if (!sv.version) {
				for (var i in version_statuses) {
					sv.status = version_statuses[i];
					if (ts_files[current_arch_std][mod][sv.status]) {
						for (var ver in ts_files[current_arch_std][mod][sv.status]) {
							sv.version = ver;
							break;
						}
					}
					if (sv.version)
						break;
				}
			}
			elem_ver.value = sv.status + '|' + sv.version;
			var disp_ver = sv.version;
			if (disp_ver == '*') {
				var tmp = ts_local_ver[current_arch_std + '|' + mod + '|' + status];
				disp_ver = tmp ? tmp : 'not installed';
			}
			elem_vis.innerHTML = disp_ver;
			elem_stat.setAttribute('src', 'images/status_' + sv.status + '.png');
			if (to_check_ts[mod]) {
				if (!elem.disabled)
					elem.checked = true;
				// Remove the test suite from the array cat.
				for (var cat in cat_list) {
					for (var i=0; i<cat_list[cat].length; ++i) {
						if (cat_list[cat][i] == mod) {
							cat_list[cat].splice(i, 1);
							break;
						}
					}
				}
			}
		}
		if (to_check_ts_opt[mod]) {
			for (var opt_name in to_check_ts_opt[mod]) {
				var opt_value = to_check_ts_opt[mod][opt_name];
				var elems = document.getElementsByName('modopt-' + current_arch_std + '-' + mod + '-' + opt_name);
				if (elems) {
					for (var i=0; i<elems.length; ++i) {
						elems[i].checked = (elems[i].value == opt_value);
					}
				}
			}
		}
	}
	for (var cat in cat_list) {
		elem = document.getElementById('check-' + cat + '-' + current_arch_std);
		if (elem) {
			elem.checked = ((cat_list[cat].length > 0) ? false : true);
		}
	}
}

////////////////////////////////////////////////////////////////////////////////
// Main Configuring and Testing Functionality

// Shows/hides the block with additional options
function show_hide_details() {
	var elem_div = document.getElementById('additional_data');
	var elem_img = document.getElementById('block_arrow_img');
	if (elem_div && elem_img) {
		var to_show = (elem_div.style.display == 'none');
		elem_div.style.display = to_show ? '' : 'none';
		elem_img.setAttribute('src', to_show ? 'images/arr_d.png' : 'images/arr_r.png');
	}
}

// Performs pre-testing checks
function do_action(name) {
	update_elements_state();
	var elem = document.getElementById('form_action');
	if (!elem) {
		alert("Internal error: Cannot find 'form_action' field!\\nPlease, contact the authors.");
		return false;
	}
	prepareSubmit();
	elem.value = name;
	if (name == 'run_tests') {
		ajax_call_post('action=check_run', 'configure_test');
	}
	return true;
}

// Saves all the dynamic data (that are not present as explicit form fields).
function prepareSubmit() {
	var tmp;
	var file_inst = document.getElementById('file_inst');
	if (file_inst) {
		tmp = '';
		// File installation type
		for (var file in ts_file_inst) {
			if (ts_file_inst[file]) {
				if (file_ts[file]) {
					// Save it for those files only that are used in current arch-std
					for (var i=0; i<file_ts[file].length; ++i) {
						if (file_ts[file][i].indexOf(current_arch_std) == 0) {
							tmp += file + ':' + ts_file_inst[file] + ':';
							break;
						}
					}
				}
			}
		}
		file_inst.value = tmp;
	}
	var file_dl = document.getElementById('file_dl');
	if (file_dl) {
		tmp = '';
		// File download type
		for (var file in ts_file_dl) {
			if (ts_file_dl[file]) {
				if (file_ts[file]) {
					// Save it for those files only that are used in current arch-std
					for (var i=0; i<file_ts[file].length; ++i) {
						if (file_ts[file][i].indexOf(current_arch_std) == 0) {
							tmp += file + ':' + ts_file_dl[file] + ':';
							break;
						}
					}
				}
			}
		}
		file_dl.value = tmp;
	}
	var auto_replies = document.getElementById('auto_replies');
	if (auto_replies) {
		tmp = '';
		for (var ts in ts_expect_info[current_arch_std]) {
			for (var status in ts_expect_info[current_arch_std][ts]) {
				for (var ver in ts_expect_info[current_arch_std][ts][status]) {
					var list = ts_expect_info[current_arch_std][ts][status][ver];
					if (list) {
						var tmp2 = ''
						for (var qid in list) {
							var qid_esc = qid.replace(/%/g, '%25').replace(/:/g, '%3A').replace(/;/g, '%3B');
							var answer_esc = list[qid].answer.replace(/%/g, '%25').replace(/:/g, '%3A').replace(/;/g, '%3B');
							tmp2 += qid_esc + ':' + answer_esc + ':';
						}
						if (tmp2 != '')
							tmp += ts + ':' + status + ':' + ver + ':' + tmp2 + ';';
					}
				}
			}
		}
		auto_replies.value = tmp;
	}
	var ts_glob_options = document.getElementById('ts_glob_options');
	if (ts_glob_options) {
		tmp = '';
		for (var ts in ts_options[current_arch_std]) {
			for (var i in ts_options[current_arch_std][ts]) {
				var opt_name = ts_options[current_arch_std][ts][i];
				var elemopt = document.getElementsByName('modopt-' + current_arch_std + '-' + ts + '-' + opt_name);
				if (elemopt) {
					var tmp2 = ''
					for (k in elemopt) {
						if (elemopt[k].checked) {
							tmp2 += opt_name.replace(/%/g, '%25').replace(/:/g, '%3A').replace(/;/g, '%3B') + ':' + elemopt[k].value.replace(/%/g, '%25').replace(/:/g, '%3A').replace(/;/g, '%3B') + ':';
							break;
						}
					}
					if (tmp2 != '')
						tmp += ts + ':' + tmp2 + ';';
				}
			}
		}
		ts_glob_options.value = tmp;
	}
}

////////////////////////////////////////////////////////////////////////////////
// Special Functions

function save_and_go(event) {
	var url = this.getAttribute('href');
	// IE fix
	if (!event)
		event = window.event;
	document.configure_test.target = (event.shiftKey ? '_blank' : '');
	document.configure_test.action = 'tests_conf_save.pl?goto=' + url.replace('#', '%23');
	prepareSubmit();
	document.configure_test.submit();
	document.configure_test.action = 'tests_conf_save.pl';
	document.configure_test.target = '';
	event.cancelBubble = true;
	return false;
}

function modify_all_links() {
	var links = document.getElementsByTagName('A');
	for (var i = 0; i < links.length; i++) {
		if ((links[i].target == '') && (links[i].href != '') && (links[i].onclick == null)) {
			links[i].onclick = save_and_go;
		}
	}
}

// Function to check for existence of nested hashes.
// Usage: check_array_elem(arr, key1, key2, ...)
// The result is true if all of the following values are defined: arr, arr[key1], arr[key1][key2], etc.
function check_array_elem() {
	if (arguments.length < 2)
		return false;
	var arr = arguments[0];
	if (!arr)
		return false;
	for (var i=1; i<arguments.length; ++i) {
		if (arr[arguments[i]])
			arr = arr[arguments[i]];
		else
			return false;
	}
	return true;
}

////////////////////////////////////////////////////////////////////////////////
//  AJAX

function onSave() {
	if (edit_profile_name.value == '') {
		edit_profile_name.style.borderColor = 'red';
		alert('Please, specify the profile name!');
	}
	else if (edit_profile_name.value.search(/[\\/\\\\<>|\\?\\*]/) != -1) {
		edit_profile_name.style.borderColor = 'red';
		alert('The profile name should contain only symbols that are allowed in file names!');
	}
	else {
		for (var i=0; i<profiles_list.length; ++i) {
			if (edit_profile_name.value == profiles_list[i]) {
				if (!confirm("Do you want to overwrite the profile '" + edit_profile_name.value + "'?"))
					return;
				break;
			}
		}
		prepareSubmit();
		ajax_call_post('action=save_profile', 'configure_test');
	}
}

function onLoad() {
	if (edit_profile_name.value) {
		ajax_call_get('action=load_profile&page=conf&profile=' + encodeURIComponent(edit_profile_name.value));
	}
	else {
		edit_profile_name.style.borderColor = 'red';
		alert('Please, specify the profile name!');
	}
}

function onDelete() {
	if (edit_profile_name.value) {
		if (confirm("Do you really want to delete the profile '" + edit_profile_name.value + "'?"))
			ajax_call_get('action=delete_profile&profile=' + encodeURIComponent(edit_profile_name.value));
	}
	else {
		edit_profile_name.style.borderColor = 'red';
		alert('Please, specify the profile name!');
	}
}

function onRefresh() {
	ajax_call_get('action=refresh_list');
}

function manifestOverwrite() {
	ajax_call_get('action=manifest_overwrite');
}

function ajaxProcessResult(responseXML) {
	edit_profile_name.value = '';
	if (responseXML.getElementsByTagName('profiles').length > 0) {
		var elem_list = document.getElementById('list_profile_name');
		var elem_div = elem_list.parentNode;
		elem_div.removeChild(elem_list);
		elem_list = document.createElement('select');
		elem_list.id = 'list_profile_name';
		elem_list.name = 'list_profile_name';
		elem_list.style.fontFamily = 'monospace';
		elem_list.style.fontSize = '100%';
		elem_list.style.width = '19.7em';
		elem_list.onclick = onClickList;
		elem_list.onkeypress = onKeyPressList;
		elem_div.appendChild(elem_list);
		list_profile_name = elem_list;
		profiles_list = new Array();

		var profile_names = responseXML.getElementsByTagName('profiles')[0].getElementsByTagName('name');
		if (profile_names.length > 0) {
			for (var i=0; i<profile_names.length; ++i) {
				profiles_list[i] = profile_names[i].childNodes[0].nodeValue;
				var tmp_node = document.createElement('option');
				tmp_node.setAttribute('value', profiles_list[i]);
				tmp_node.appendChild(document.createTextNode(profiles_list[i]));
				elem_list.appendChild(tmp_node);
			}
			elem_list.disabled = false;
			if (profile_names.length < 2)
				elem_list.size = 2;
			else if (profile_names.length > 6)
				elem_list.size = 6;
			else
				elem_list.size = profile_names.length;
		}
		else {
			var tmp_node = document.createElement('option');
			tmp_node.setAttribute('value', '');
			tmp_node.appendChild(document.createTextNode('<No profiles present>'));
			elem_list.appendChild(tmp_node);
			elem_list.disabled = true;
			elem_list.size = 2;
		}
	}
	if (responseXML.getElementsByTagName('js').length > 0) {
		var js_data = '';
		for (var i=0; i<responseXML.getElementsByTagName('js').length; ++i) {
			for (var j=0; j<responseXML.getElementsByTagName('js')[i].childNodes.length; ++j)
				js_data += responseXML.getElementsByTagName('js')[i].childNodes[j].nodeValue;
		}
		eval(js_data);
	}
	if (responseXML.getElementsByTagName('modlist').length > 0) {
		var modlist_data = responseXML.getElementsByTagName('modlist')[0];
		var modlist_block = document.getElementById('modlist_block');
		if (modlist_block) {
			save_profile_options_modules();
			var html_data = '';
			for (var i=0; i<modlist_data.getElementsByTagName('html').length; ++i) {
				for (var j=0; j<modlist_data.getElementsByTagName('html')[i].childNodes.length; ++j)
					html_data += modlist_data.getElementsByTagName('html')[i].childNodes[j].nodeValue;
			}
			modlist_block.innerHTML = html_data;
			// Construct the Standard listbox
			var elem_list = document.getElementById('std_version');
			var selected_stdv = elem_list.value;
			var elem_parent = elem_list.parentNode;
			elem_parent.removeChild(elem_list);
			elem_list = document.createElement('select');
			elem_list.id = 'std_version';
			elem_list.name = 'std_version';
			elem_list.style.width = '9em';
			elem_list.onchange = show_modules;
			elem_parent.appendChild(elem_list);
			var group = '';
			var group_elem;
			var new_stdv = '';
			for (var i in stdv_list) {
				var stdv = stdv_list[i].replace(/:/g, ' ');
				if (stdv == selected_stdv)
					new_stdv = selected_stdv;
				var pos = stdv.indexOf(' ');
				var newgroup = ((pos >= 0) ? stdv.substr(0, pos) : 'Unknown');
				if (group != newgroup) {
					group_elem = document.createElement('optgroup');
					group_elem.label = newgroup;
					elem_list.appendChild(group_elem);
					group = newgroup;
				}
				var tmp_node = document.createElement('option');
				tmp_node.setAttribute('value', stdv);
				tmp_node.appendChild(document.createTextNode(stdv));
				group_elem.appendChild(tmp_node);
			}
			if (new_stdv == '')
				new_stdv = '$default_profile_general{'STD_VERSION'}';
			elem_list.value = new_stdv;

			var outdated_warn = document.getElementById('outdated_warn');
			if (outdated_warn)
				outdated_warn.style.display = 'none';
			var refresh_button = document.getElementById('refresh_list_button');
			if (refresh_button) {
				refresh_button.value = '';      // Else font-width style is not applied...
				refresh_button.value = 'Refresh List';
				refresh_button.style.fontWeight = 'normal';
				refresh_button.style.color = 'black';
			}
			apply_profile_options_modules();
			show_modules();
		}
	}
	if (responseXML.getElementsByTagName('testrun_allowed').length > 0) {
		var testrun_allowed = responseXML.getElementsByTagName('testrun_allowed')[0];
		if (testrun_allowed)
			document.configure_test.submit();
	}
}

////////////////////////////////////////////////////////////////////////////////
// Initialization

init_data();
modify_all_links();
set_options_by_profile();
// ]]>
</script>
DATA

print_footer();
