#!/usr/bin/perl -w

# Distribution Checker
# Tests Execution Module (tests_exec.pl)
#
# Copyright (C) 2007-2009 The Linux Foundation. All rights reserved.
#
# This program has been developed by ISP RAS for LF.
# The ptyshell tool is originally written by Jiri Dluhos <jdluhos@suse.cz>
# Copyright (C) 2005-2007 SuSE Linux Products GmbH
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# version 2 as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

use Templates;
use TestStatus;

my $js_init = '';
if ($_COOKIE{'profile'}) {
	# Start tests via AJAX
	$js_init = "startTests('$_COOKIE{'profile'}');\n";
}
else {
	my $status = read_status();
	if ($status->{'IS_RUNNING'}) {
		$js_init = "cert = '".($status->{'CERTIFICATION'} ? 'certification' : 'custom')."';\nstartTestsPrepareGUI();\nstartRefresh();\n";
	}
}

print "HTTP/1.0 200 OK" . CRLF;
print "Content-type: text/html" . CRLF;
print "Set-Cookie: profile=; path=/;" . CRLF . CRLF;

print_header('Test Execution Console Page', 'exec');

print show_error_dlg('');

print <<DATA;

<div id="ajax_loading" style="display: none;"></div>

<h1>Test Execution</h1>
Please, select the user profile to run:&nbsp;&nbsp;&nbsp;
DATA

my $found = 0;
my $profiles_list = '';
if (opendir(DIR, $SERVER_PARAM{'APP_DATA'}.'/profiles')) {
	my @files = sort grep !/^[\.~]/, readdir(DIR);
	$found = scalar(@files);
	foreach (@files) {
		if ($_COOKIE{'profile'} and ($_COOKIE{'profile'} eq $_)) {
			$profiles_list .= "    <option value=\"$_\" selected=\"selected\">$_</option>\n";
		}
		else {
			$profiles_list .= "    <option value=\"$_\">$_</option>\n";
		}
	}
	closedir(DIR);
}

if ($found) {
	print <<DATA;
  <select name="test_profile" id="test_profile" style="width: 12em;">
$profiles_list  </select>&nbsp;
  <input type="button" name="Start" id="start_button" value="Start" style="width: 6em;" onclick="javascript:startTests('');" />&nbsp;
  <input type="button" name="Stop" id="stop_button" value="Stop" style="width: 6em;" onclick="javascript:stopTests();" disabled="disabled" />
DATA
}
else {
	print <<DATA;
  <select name="test_profile_no" id="test_profile_no" style="width: 12em;" disabled="disabled">
    <option value="">&lt;No profiles present&gt;</option>
  </select>&nbsp;
  <input type="button" name="Start" id="start_button_no" value="Start" style="width: 6em;" disabled="disabled" />&nbsp;
  <input type="button" name="Stop" id="stop_button" value="Stop" style="width: 6em;" onclick="javascript:stopTests();" disabled="disabled" />
DATA
}

print <<DATA;
<br /><br />
<table border="0" cellpadding="0" cellspacing="0" style="width: 60em;"><tr><td align="left">
<b><font color="blue"><span id="exec_info">Nothing started</span></font>&nbsp;<span id="exec_status"></span></b> <b><a id="go_to_manual" style="display: none;" href="tests_appbat.pl">Run manual tests</a></b>
</td><td align="right">
<b><a id="view_report" style="display: none;" href="tests_results.pl">View test report</a></b>
</td></tr></table>

<pre id="cmdlog" style="height: 400px; width: 60em; overflow: auto; text-wrap: none; border: 1px solid black; background-color: black; color: lightgreen;">Execute output will go here&hellip;</pre>

<div id="progress_table" style="border-bottom: solid 1px #aaaaaa; position: absolute; top: 250px; left: 850px; opacity: 1.0; filter: alpha(opacity=100);" onmouseover="javascript:setOpacity(this, 1.0);" onmouseout="javascript:setOpacity(this, 0.7);">
</div>

<script language="javascript" type="text/javascript">
// <![CDATA[
// Preload images for shadow, various icons, buttons, etc.
preload_images(new Array('ajax_progress.gif', 'install_progress.gif', 'test_failed.png', 'test_warnings.png', 'test_passed.png', 'test_no_verdict.png', 'test_incomplete.png', 'test_crashed.png', 'progress.png', 'progress_finished.png', 'progress_crashed.png', 'progress_divider.png'));

var cmdlog = document.getElementById('cmdlog');
var exec_status = document.getElementById('exec_status');
var exec_info = document.getElementById('exec_info');
var go_to_manual = document.getElementById('go_to_manual');
var view_report = document.getElementById('view_report');
var progress_table = document.getElementById('progress_table');
if (!cmdlog || !exec_status || !exec_info || !go_to_manual || !view_report || !progress_table)
	alert('Internal error: Cannot find essential form fields! JavaScript will not work correctly.\\nPlease, contact the authors.');

var start_button = document.getElementById('start_button');
var stop_button = document.getElementById('stop_button');
var test_profile = document.getElementById('test_profile');

var cmdlog_height;
if (navigator.appName.indexOf('Opera') != -1)
	cmdlog_height = cmdlog.style.pixelHeight;
else
	cmdlog_height = cmdlog.clientHeight;

var cert = '';
var log_contents = '';
var last_line = '';

// If the element intersects the console log, the function changes its opacity as requested
function setOpacity(elem, opacity_value) {
	var pos_elem = getPosition(elem);
	var drag_rect = {l: pos_elem.x, r: pos_elem.x + elem.offsetWidth, t: pos_elem.y, b: pos_elem.y + elem.offsetHeight};
	var pos_cmdlog = getPosition(cmdlog);
	var cons_rect = {l: pos_cmdlog.x, r: pos_cmdlog.x + cmdlog.offsetWidth, t: pos_cmdlog.y, b: pos_cmdlog.y + cmdlog.offsetHeight};
	if ((cons_rect.l <= drag_rect.r) && (drag_rect.l <= cons_rect.r) && (cons_rect.t <= drag_rect.b) && (drag_rect.t <= cons_rect.b)) {
		elem.style.opacity = opacity_value;
		// IE opacity support
		elem.style.filter = 'alpha(opacity=' + (opacity_value * 100) + ')';
	}
}

function format_time(tm) {
	var h = Math.floor(tm / 3600);
	var m = Math.floor(tm / 60 - h * 60);
	var s = tm - h * 3600 - m * 60;
	return h + ':' + ((m < 10) ? '0' + m : m) + ':' + ((s < 10) ? '0' + s : s);
}

// AJAX
var refresh_delay = 1000;
timeout_show_progress = -1;
var timeout_var;
var test_timer_var;

var progress_table_present = false;
var current_test_id = '';
var current_test_time = 0;
var server_time_estimated = 0;

function updateTestTimer() {
	var next_call_period = 1000;
	if (current_test_id) {
		var elem = document.getElementById(current_test_id + '-elapsed-time');
		if (elem) {
			++current_test_time;
			++server_time_estimated;
			elem.innerHTML = format_time(current_test_time);
			// Correct the speed of internal time counter to catch up the server time
			var dt = server_time_estimated - current_test_time;
			// Heuristic function for adjusting timer speed.
			// Time to the next timer call is F(dt) seconds, where:
			// F(x) = 2                         , if x in (-oo, -4);
			// F(x) = 1 - 3*|x|*x/64 + x^4/1024 , if x in [-4, 4];
			// F(x) = 0.5                       , if x in (4, +oo).
			if (dt < -4) {
				next_call_period = 2000;
			}
			else if (dt > 4) {
				next_call_period = 500;
			}
			else {
				var z = Math.abs(dt) * dt / 16;
				next_call_period = Math.round((4 - 3 * z + z * z) * 250);
			}
		}
	}
	test_timer_var = setTimeout('updateTestTimer()', next_call_period);
}

function startTestsPrepareGUI() {
	if (start_button)
		start_button.disabled = true;
	if (stop_button)
		stop_button.disabled = false;
	if (test_profile)
		test_profile.disabled = true;
	exec_info.innerHTML = 'Preparing to run the tests&hellip;';
	exec_status.innerHTML = '&nbsp;&nbsp;&nbsp;<img src="images/ajax_progress.gif" width="16" height="16" alt="" />';
	cmdlog.innerHTML = '';
	cmdlog.style.color = 'lightgreen';
	log_contents = '';
	last_line = '';
	current_test_time = 0;
	server_time_estimated = 0;
	current_test_id = '';
}

function startTests(profile_name) {
	if (!profile_name) {
		var profile = document.getElementById('test_profile');
		if (profile) {
			if (profile.value) {
				profile_name = profile.value;
			}
			else {
				alert('Profile is not specified!');
				return;
			}
		}
		else {
			alert("Internal error: Cannot find 'profile' form field!\\nPlease, contact the authors.!");
			return;
		}
	}
	startTestsPrepareGUI();
	ajax_call_get('action=run_tests&profile=' + encodeURIComponent(profile_name));
}

function stopTestsPrepareGUI() {
	if (start_button)
		start_button.disabled = false;
	if (stop_button)
		stop_button.disabled = true;
	if (test_profile)
		test_profile.disabled = false;
}

function stopTests() {
	if (confirm('Are you sure you want to terminate test execution?')) {
		if (stop_button)
			stop_button.disabled = true;
		exec_info.innerHTML = 'Tests are stopping&hellip;';
		ajax_call_get('action=stop_tests&tree=tests');
	}
}

function startRefresh() {
	exec_info.innerHTML = 'Tests are running (' + cert + ')&hellip;';
	exec_status.innerHTML = '&nbsp;&nbsp;&nbsp;<img src="images/ajax_progress.gif" width="16" height="16" alt="" />';
	cmdlog.innerHTML = '';
	cmdlog.style.color = 'lightgreen';
	log_contents = '';
	last_line = '';
	ajax_call_get('action=get_test_log&start=0');
	updateTestTimer();
}

function stopRefresh() {
	clearTimeout(timeout_var);
	timeout_var = null;
	clearTimeout(test_timer_var);
	test_timer_var = null;
	stopTestsPrepareGUI();
	progress_table_present = false;
	current_test_id = '';
	cmdlog.style.color = '#ffa0b0';
}

function onAjaxError() {
	clearTimeout(timeout_var);
	timeout_var = null;
	clearTimeout(test_timer_var);
	test_timer_var = null;
	stopTestsPrepareGUI();
	exec_info.innerHTML = 'Nothing started';
	exec_status.innerHTML = '';
}

var test_progress_data;

var status_images = new Array();
status_images['Not started'] = 'images/environment-spacer.gif';
status_images['Preparing'] = 'images/install_progress.gif';
status_images['Running'] = 'images/ajax_progress.gif';
status_images['Making report'] = 'images/ajax_progress.gif';
status_images['Failed'] = 'images/test_failed.png';
status_images['Warnings'] = 'images/test_warnings.png';
status_images['Passed'] = 'images/test_passed.png';
status_images['No verdict'] = 'images/test_no_verdict.png';
status_images['Incomplete'] = 'images/test_incomplete.png';
status_images['Crashed'] = 'images/test_crashed.png';

var progress_images = new Array();
progress_images['Not started'] = 'images/progress.png';
progress_images['Preparing'] = 'images/progress.png';
progress_images['Running'] = 'images/progress.png';
progress_images['Making report'] = 'images/progress.png';
progress_images['Failed'] = 'images/progress_finished.png';
progress_images['Warnings'] = 'images/progress_finished.png';
progress_images['Passed'] = 'images/progress_finished.png';
progress_images['No verdict'] = 'images/progress_finished.png';
progress_images['Incomplete'] = 'images/progress_finished.png';
progress_images['Crashed'] = 'images/progress_crashed.png';

var time_colors = new Array();
time_colors['Not started'] = 'gray';
time_colors['Preparing'] = 'blue';
time_colors['Running'] = 'blue';
time_colors['Making report'] = 'blue';
time_colors['Failed'] = 'black';
time_colors['Warnings'] = 'black';
time_colors['Passed'] = 'black';
time_colors['No verdict'] = 'black';
time_colors['Incomplete'] = 'black';
time_colors['Crashed'] = 'darkred';

// Combines statuses into classes by the state of the test execution progress
function convertStatusToState(status) {
	if ((status == 'Not started') || (status == 'Crashed'))
		return status;
	if ((status == 'Preparing') || (status == 'Running') || (status == 'Making report'))
		return 'Running';
	else if ((status == 'Failed') || (status == 'Warnings') || (status == 'Passed') || (status == 'No verdict') || (status == 'Incomplete'))
		return 'Finished';
	else
		return 'Unknown';
}

function ajaxProcessResult(responseXML) {
	var tid;
	if (responseXML.getElementsByTagName('cert').length > 0) {
		cert = ((responseXML.getElementsByTagName('cert')[0].childNodes[0].nodeValue == '0') ? 'custom' : 'certification');
	}
	if (responseXML.getElementsByTagName('progress').length > 0) {
		var progress_data = responseXML.getElementsByTagName('progress')[0];
		var testsuites_data = progress_data.getElementsByTagName('testsuite');
		if (!progress_table_present) {
			// Build the progress table HTML code
			test_progress_data = new Array();
			var html_data = '<table class="progress-table" cellpadding="0" cellspacing="0" style="width: 26em; background: #f2f2f2;" onmousedown="javascript:dragStart(event, progress_table, true);" onmouseup="javascript:dragStop(event);">\\n  <tr>\\n    <th style="width: 26px; padding: 0; text-align: center;"><img src="images/environment-spacer.gif" alt="" width="16" height="16" /></th>\\n    <th style="width: 13em;">Test Suite</th>\\n    <th style="width: 5em;">Elapsed</th>\\n    <th style="width: 5em;">Estimated</th>\\n  </tr>\\n</table>\\n<div style="max-height: 400px; overflow: auto;">\\n  <table class="progress-table" cellpadding="0" cellspacing="0" style="width: 26em; background: #f9f9f9; border-top: none; border-bottom: none;" onmousedown="javascript:dragStart(event, progress_table, true);" onmouseup="javascript:dragStop(event);">';
			if (testsuites_data.length == 0) {
				html_data += '   <tr>\\n      <td colspan="4" style="border-top: none; color: gray; text-align: center;">Nothing started</td>\\n    </tr>\\n';
			}
			else {
				for (var i = 0; i < testsuites_data.length; ++i) {
					var id = testsuites_data[i].getAttribute('id');
					var name = testsuites_data[i].getElementsByTagName('name')[0].childNodes[0].nodeValue;
					var status = testsuites_data[i].getElementsByTagName('status')[0].childNodes[0].nodeValue;
					var elapsed = testsuites_data[i].getElementsByTagName('elapsed')[0].childNodes[0].nodeValue;
					var total = testsuites_data[i].getElementsByTagName('total')[0].childNodes[0].nodeValue;
					var prepare_percent = testsuites_data[i].getElementsByTagName('prep_percent')[0].childNodes[0].nodeValue;
					var percent = testsuites_data[i].getElementsByTagName('percent')[0].childNodes[0].nodeValue;

					var total_time = '-:--:--';
					var elapsed_time = '-:--:--';
					var name_style = '';
					var state = convertStatusToState(status);

					if (state == 'Not started') {
						percent = 0;
						if (total != 0)
							total_time = format_time(total);
					}
					else if (state == 'Running') {
						current_test_id = id;
						server_time_estimated = elapsed;
						current_test_time = elapsed;
						elapsed_time = format_time(elapsed);
						if (total != 0)
							total_time = format_time(total);
						name_style = ' style="font-weight: bold; color: blue;"';
					}
					else if (state == 'Finished') {
						if (current_test_id == id)
							current_test_id = '';
						percent = 100;
						elapsed_time = format_time(elapsed);
						total = elapsed;
						total_time = elapsed_time;
					}
					else if (state == 'Crashed') {
						if (current_test_id == id)
							current_test_id = '';
						elapsed_time = format_time(elapsed);
						total_time = format_time(total);
					}

					test_progress_data[id] = new Array();
					test_progress_data[id]['name'] = name;
					test_progress_data[id]['status'] = status;
					test_progress_data[id]['elapsed'] = elapsed;
					test_progress_data[id]['total'] = total;
					test_progress_data[id]['prepare_percent'] = prepare_percent;
					test_progress_data[id]['percent'] = percent;

					var prep_progress_width = Math.round(prepare_percent * 1.5);
					var progress_width = Math.round(percent * 1.5);
					var progress_img = '';
					if (prep_progress_width) {
						progress_img += '<img id="' + id + '-prep-progress-bar" src="' + progress_images[status] + '" alt="' + prepare_percent + '%" width="' + prep_progress_width + '" height="5" /><img id="' + id + '-div-progress-bar" src="images/progress_divider.png" alt="" width="1" height="5" />';
						progress_width -= prep_progress_width + 1;
						if (progress_width < 0)
							progress_width = 0;
					}
					else {
						progress_img += '<img id="' + id + '-prep-progress-bar" src="' + progress_images[status] + '" alt="0%" width="0" height="5" style="display: none;" /><img id="' + id + '-div-progress-bar" src="images/progress_divider.png" alt="" width="0" height="5" style="display: none;" />';
					}
					if (progress_width) {
						progress_img += '<img id="' + id + '-progress-bar" src="' + progress_images[status] + '" alt="' + percent + '%" width="' + progress_width + '" height="5" />';
					}
					else {
						progress_img += '<img id="' + id + '-progress-bar" src="' + progress_images[status] + '" alt="0%" width="0" height="5" style="display: none;" />';
					}

					var border = '';
					if (i == 0)
						border += ' border-top: none;';
					if (i == testsuites_data.length - 1)
						border += ' border-bottom: none;';
					html_data += '  <tr>\\n    <td style="width: 26px; padding: 0; text-align: center;' + border + '"><img id="' + id + '-img" src="' + status_images[status] + '" alt="' + status + '" title="' + status + '" width="16" height="16" /></td>\\n    <td id="' + id + '" style="width: 13em;' + border + '">\\n      <span id="' + id + '-name"' + name_style + '>' + name + '</span>\\n      <div style="width: 150px; height: 5px; border: 1px solid #004000; text-align: left;">' + progress_img + '</div>\\n    </td>\\n    <td style="width: 5em;' + border + ' white-space: nowrap; text-align: right;"><span id="' + id + '-elapsed-time" style="color: ' + time_colors[status] + '; font-family: monospace;">' + elapsed_time + '</span></td>\\n    <td style="width: 5em;' + border + ' white-space: nowrap; text-align: right;"><span id="' + id + '-total-time" style="color: ' + time_colors[status] + '; font-family: monospace;">' + total_time + '</span></td>\\n  </tr>\\n';
				}
			}
			html_data += '  </table>\\n</div>';
			progress_table.innerHTML = html_data;
			var pos_str = getCookie('pos_progress_table');
			if (pos_str) {
				var comma_idx = pos_str.indexOf(',');
				progress_table.style.left = pos_str.substr(0, comma_idx);
				progress_table.style.top = pos_str.substr(comma_idx + 1);
			}
			progress_table.style.display = '';
			// If the list is long enough, the scrollbar is shown.
			// In this case we increase the width so that the scrollbar was outside the table
			if (progress_table.offsetHeight >= 400)
				progress_table.style.width = (progress_table.offsetWidth + 17) + 'px';
			setOpacity(progress_table, 0.7); // Set opacity, if the table intersects with cmdlog area
			progress_table_present = true;
		}
		else {
			// Update the progress table data
			for (var i = 0; i < testsuites_data.length; ++i) {
				var id = testsuites_data[i].getAttribute('id');
				var status = testsuites_data[i].getElementsByTagName('status')[0].childNodes[0].nodeValue;
				var elapsed = testsuites_data[i].getElementsByTagName('elapsed')[0].childNodes[0].nodeValue;
				var total = testsuites_data[i].getElementsByTagName('total')[0].childNodes[0].nodeValue;
				var prepare_percent = testsuites_data[i].getElementsByTagName('prep_percent')[0].childNodes[0].nodeValue;
				var percent = testsuites_data[i].getElementsByTagName('percent')[0].childNodes[0].nodeValue;

				var total_time = '-:--:--';
				var elapsed_time = '-:--:--';
				var state = convertStatusToState(status);

				var name_elem = document.getElementById(id + '-name');
				var img_elem = document.getElementById(id + '-img');
				var prep_percent_elem = document.getElementById(id + '-prep-progress-bar');
				var div_percent_elem = document.getElementById(id + '-div-progress-bar');
				var percent_elem = document.getElementById(id + '-progress-bar');
				var elapsed_time_elem = document.getElementById(id + '-elapsed-time');
				var total_time_elem = document.getElementById(id + '-total-time');
				if (!name_elem || !img_elem || !prep_percent_elem || !div_percent_elem || !percent_elem || !elapsed_time_elem || !total_time_elem)
					continue;

				// Avoid flickering from unnecessary refreshes
				if (test_progress_data[id]['status'] != status) {
					img_elem.setAttribute('src', status_images[status]);
					img_elem.setAttribute('alt', status);
					img_elem.setAttribute('title', status);
					elapsed_time_elem.style.color = time_colors[status];
					total_time_elem.style.color = time_colors[status];
					if (state == 'Running') {
						current_test_id = id;
						server_time_estimated = elapsed;
						current_test_time = elapsed;
						name_elem.style.fontWeight = 'bold';
						name_elem.style.color = 'blue';
					}
					else {
						name_elem.style.fontWeight = 'normal';
						name_elem.style.color = 'black';
					}
				}

				if (state == 'Not started') {
					percent = 0;
					if (total != 0)
						total_time = format_time(total);
				}
				else if (state == 'Running') {
					elapsed_time = format_time(elapsed);
					if (total != 0)
						total_time = format_time(total);
				}
				else if (state == 'Finished') {
					if (current_test_id == id)
						current_test_id = '';
					percent = 100;
					elapsed_time = format_time(elapsed);
					total = elapsed;
					total_time = elapsed_time;
				}
				else if (state == 'Crashed') {
					if (current_test_id == id)
						current_test_id = '';
					elapsed_time = format_time(elapsed);
					total_time = format_time(total);
				}

				test_progress_data[id]['status'] = status;
				test_progress_data[id]['elapsed'] = elapsed;
				test_progress_data[id]['total'] = total;
				test_progress_data[id]['prepare_percent'] = prepare_percent;
				test_progress_data[id]['percent'] = percent;

				var prep_progress_width = Math.round(prepare_percent * 1.5);
				var progress_width = Math.round(percent * 1.5);

				prep_percent_elem.setAttribute('src', progress_images[status]);
				prep_percent_elem.setAttribute('alt', prepare_percent + '%');
				prep_percent_elem.width = prep_progress_width;
				if (prep_progress_width) {
					prep_percent_elem.style.display = '';
					div_percent_elem.width = 1;
					div_percent_elem.style.display = '';
					progress_width -= prep_progress_width + 1;
					if (progress_width < 0)
						progress_width = 0;
				}
				else {
					prep_percent_elem.style.display = 'none';
					div_percent_elem.width = 0;
					div_percent_elem.style.display = 'none';
				}

				percent_elem.setAttribute('src', progress_images[status]);
				percent_elem.setAttribute('alt', percent + '%');
				percent_elem.width = progress_width;
				percent_elem.style.display = ((progress_width == 0) ? 'none' : '');
				if (state != 'Running')
					elapsed_time_elem.innerHTML = elapsed_time;
				total_time_elem.innerHTML = total_time;
			}
		}
	}
	if (responseXML.getElementsByTagName('started').length > 0) {
		setTimeout('startRefresh()', 100);
	}
	else {
		if (responseXML.getElementsByTagName('output').length > 0) {
			var output = '';
			for (var i=0; i<responseXML.getElementsByTagName('output').length; ++i) {
				for (var j=0; j<responseXML.getElementsByTagName('output')[i].childNodes.length; ++j)
					output += responseXML.getElementsByTagName('output')[i].childNodes[j].nodeValue;
			}
			updateCmdLog(output);
		}
		if (responseXML.getElementsByTagName('tid').length > 0) {
			tid = responseXML.getElementsByTagName('tid')[0].childNodes[0].nodeValue;
		}
		if (responseXML.getElementsByTagName('size').length > 0) {
			var sz = responseXML.getElementsByTagName('size')[0].childNodes[0].nodeValue;
			timeout_var = setTimeout("ajax_call_get('action=get_test_log&start=" + sz + "&tid=" + tid + "')", refresh_delay);
		}
		else {
			if (responseXML.getElementsByTagName('tr_status').length > 0) {
				exec_info.innerHTML = 'Tests are finished (' + cert + ').';
				exec_status.innerHTML = responseXML.getElementsByTagName('tr_status')[0].childNodes[0].nodeValue;
				stopRefresh();
			}
			if (responseXML.getElementsByTagName('redirect').length > 0) {
				document.location = 'tests_results.pl?details=' + responseXML.getElementsByTagName('redirect')[0].childNodes[0].nodeValue + '&summary=1';
			}
		}
	}
}

var re_x0d = new RegExp();
re_x0d.compile('^[^\\x0d]*\\x0d', 'g');

function make_merge(line) {
	var tmp;
	// Make every ^M control character (\\x0d) erase all from the beginning of line
	while (1) {
		re_x0d.lastIndex = 0;
		tmp = line.replace(re_x0d, '');
		if (tmp != line)
			line = tmp;
		else
			break;
	}
	return line;
}

function updateCmdLog(txt) {
	var scroll_top = cmdlog.scrollTop;
	var scroll_height = cmdlog.scrollHeight;
	var auto_scroll = (scroll_height - cmdlog_height - scroll_top < 10);
	last_line += txt;
	var fnewln_pos = last_line.indexOf('\\n');
	var lnewln_pos = last_line.lastIndexOf('\\n');

	if (fnewln_pos == -1) {
		// merge and make new last_line
		last_line = make_merge(last_line);
	}
	else {
		// block1 (till first '\\n'): for merging and adding to log_contents
		// block2 (from first till last '\\n'): for adding to log_contents (no merging needed)
		// block3 (after last '\\n'): to be the new last_line (no merging needed)
		log_contents += make_merge(last_line.substring(0, fnewln_pos + 1)) + last_line.substring(fnewln_pos + 1, lnewln_pos + 1);
		last_line = last_line.substring(lnewln_pos + 1);
	}
	if (typeof(cmdlog.innerText) == 'undefined')
		cmdlog.textContent = log_contents + last_line;
	else
		cmdlog.innerText = log_contents + last_line;
	if (auto_scroll) {
		cmdlog.scrollTop = cmdlog.scrollHeight;
		cmdlog.scrollTop = cmdlog.scrollHeight;		// IE fix
	}
}

$js_init
// ]]>
</script>
DATA

print_footer();
